@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,12 +20,14 @@
 #define CPPCHECKEXECUTOR_H
 
 #include "errorlogger.h"
+#include <cstdio>
 #include <ctime>
 #include <set>
 #include <string>
 
 class CppCheck;
 class Settings;
+class Library;
 
 /**
  * This class works as an example of how CppCheck can be used in external
@@ -87,6 +89,21 @@ public:
      */
     static void reportStatus(std::size_t fileindex, std::size_t filecount, std::size_t sizedone, std::size_t sizetotal);
 
+    /**
+     * @param fp Output file
+     */
+    static void setExceptionOutput(FILE* fp);
+    /**
+    * @return file name to be used for output from exception handler. Has to be either "stdout" or "stderr".
+    */
+    static FILE* getExceptionOutput();
+
+    /**
+    * Tries to load a library and prints warning/error messages
+    * @return false, if an error occured (except unknown XML elements)
+    */
+    static bool tryLoadLibrary(Library& destination, const char* basepath, const char* filename);
+
 protected:
 
     /**
@@ -108,6 +125,30 @@ protected:
 
 private:
 
+    /**
+     * Wrapper around check_internal
+     *   - installs optional platform dependent signal handling
+     *
+     * * @param cppcheck cppcheck instance
+    * @param argc from main()
+    * @param argv from main()
+     **/
+    int check_wrapper(CppCheck& cppcheck, int argc, const char* const argv[]);
+
+    /**
+    * Starts the checking.
+    *
+    * @param cppcheck cppcheck instance
+    * @param argc from main()
+    * @param argv from main()
+    * @return EXIT_FAILURE if arguments are invalid or no input files
+    *         were found.
+    *         If errors are found and --error-exitcode is used,
+    *         given value is returned instead of default 0.
+    *         If no errors are found, 0 is returned.
+    */
+    int check_internal(CppCheck& cppcheck, int argc, const char* const argv[]);
+
     /**
      * Pointer to current settings; set while check() is running.
      */
@@ -128,6 +169,11 @@ private:
      */
     std::time_t time1;
 
+    /**
+     * Output file name for exception handler
+     */
+    static FILE* exceptionOutput;
+
     /**
      * Has --errorlist been given?
      */
