@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,9 +19,12 @@
 #include "threadexecutor.h"
 #include "cppcheck.h"
 #include "cppcheckexecutor.h"
+#include <iostream>
+#ifdef __SVR4  // Solaris
+#include <sys/loadavg.h>
+#endif
 #ifdef THREADING_MODEL_FORK
 #include <algorithm>
-#include <iostream>
 #include <sys/select.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -116,7 +119,7 @@ int ThreadExecutor::handleRead(int rpipe, unsigned int &result)
 
         if (!_settings.nomsg.isSuppressed(msg._id, file, line)) {
             // Alert only about unique errors
-            std::string errmsg = msg.toString(_settings._verbose);
+            std::string errmsg = msg.toString(_settings.verbose);
             if (std::find(_errorList.begin(), _errorList.end(), errmsg) == _errorList.end()) {
                 _errorList.push_back(errmsg);
                 if (type == REPORT_ERROR)
@@ -138,6 +141,26 @@ int ThreadExecutor::handleRead(int rpipe, unsigned int &result)
     return 1;
 }
 
+bool ThreadExecutor::checkLoadAverage(size_t nchildren)
+{
+#if defined(__CYGWIN__) || defined(__QNX__)  // getloadavg() is unsupported on Cygwin, Qnx.
+    return true;
+#else
+    if (!nchildren || !_settings.loadAverage) {
+        return true;
+    }
+
+    double sample(0);
+    if (getloadavg(&sample, 1) != 1) {
+        // disable load average checking on getloadavg error
+        return true;
+    } else if (sample < _settings.loadAverage) {
+        return true;
+    }
+    return false;
+#endif
+}
+
 unsigned int ThreadExecutor::check()
 {
     _fileCount = 0;
@@ -155,7 +178,8 @@ unsigned int ThreadExecutor::check()
     std::map<std::string, std::size_t>::const_iterator i = _files.begin();
     for (;;) {
         // Start a new child
-        if (i != _files.end() && rpipes.size() < _settings._jobs) {
+        size_t nchildren = rpipes.size();
+        if (i != _files.end() && nchildren < _settings.jobs && checkLoadAverage(nchildren)) {
             int pipes[2];
             if (pipe(pipes) == -1) {
                 std::cerr << "pipe() failed: "<< std::strerror(errno) << std::endl;
@@ -211,8 +235,10 @@ unsigned int ThreadExecutor::check()
             FD_ZERO(&rfds);
             for (std::list<int>::const_iterator rp = rpipes.begin(); rp != rpipes.end(); ++rp)
                 FD_SET(*rp, &rfds);
-
-            int r = select(*std::max_element(rpipes.begin(), rpipes.end()) + 1, &rfds, NULL, NULL, NULL);
+            struct timeval tv; // for every second polling of load average condition
+            tv.tv_sec = 1;
+            tv.tv_usec = 0;
+            int r = select(*std::max_element(rpipes.begin(), rpipes.end()) + 1, &rfds, nullptr, nullptr, &tv);
 
             if (r > 0) {
                 std::list<int>::iterator rp = rpipes.begin();
@@ -233,7 +259,7 @@ unsigned int ThreadExecutor::check()
 
                             _fileCount++;
                             processedsize += size;
-                            if (!_settings._errorsOnly)
+                            if (!_settings.quiet)
                                 CppCheckExecutor::reportStatus(_fileCount, _files.size(), processedsize, totalfilesize);
 
                             close(*rp);
@@ -322,7 +348,7 @@ void ThreadExecutor::addFileContent(const std::string &path, const std::string &
 
 unsigned int ThreadExecutor::check()
 {
-    HANDLE *threadHandles = new HANDLE[_settings._jobs];
+    HANDLE *threadHandles = new HANDLE[_settings.jobs];
 
     _itNextFile = _files.begin();
 
@@ -338,15 +364,15 @@ unsigned int ThreadExecutor::check()
     InitializeCriticalSection(&_errorSync);
     InitializeCriticalSection(&_reportSync);
 
-    for (unsigned int i = 0; i < _settings._jobs; ++i) {
-        threadHandles[i] = (HANDLE)_beginthreadex(NULL, 0, threadProc, this, 0, NULL);
+    for (unsigned int i = 0; i < _settings.jobs; ++i) {
+        threadHandles[i] = (HANDLE)_beginthreadex(nullptr, 0, threadProc, this, 0, nullptr);
         if (!threadHandles[i]) {
             std::cerr << "#### .\nThreadExecutor::check error, errno :" << errno << std::endl;
             exit(EXIT_FAILURE);
         }
     }
 
-    DWORD waitResult = WaitForMultipleObjects(_settings._jobs, threadHandles, TRUE, INFINITE);
+    DWORD waitResult = WaitForMultipleObjects(_settings.jobs, threadHandles, TRUE, INFINITE);
     if (waitResult != WAIT_OBJECT_0) {
         if (waitResult == WAIT_FAILED) {
             std::cerr << "#### .\nThreadExecutor::check wait failed, result: " << waitResult << " error: " << GetLastError() << std::endl;
@@ -358,7 +384,7 @@ unsigned int ThreadExecutor::check()
     }
 
     unsigned int result = 0;
-    for (unsigned int i = 0; i < _settings._jobs; ++i) {
+    for (unsigned int i = 0; i < _settings.jobs; ++i) {
         DWORD exitCode;
 
         if (!GetExitCodeThread(threadHandles[i], &exitCode)) {
@@ -396,15 +422,10 @@ unsigned int __stdcall ThreadExecutor::threadProc(void *args)
     CppCheck fileChecker(*threadExecutor, false);
     fileChecker.settings() = threadExecutor->_settings;
 
-    LeaveCriticalSection(&threadExecutor->_fileSync);
-
     for (;;) {
-
-        EnterCriticalSection(&threadExecutor->_fileSync);
-
         if (it == threadExecutor->_files.end()) {
             LeaveCriticalSection(&threadExecutor->_fileSync);
-            return result;
+            break;
 
         }
         const std::string &file = it->first;
@@ -426,15 +447,12 @@ unsigned int __stdcall ThreadExecutor::threadProc(void *args)
 
         threadExecutor->_processedSize += fileSize;
         threadExecutor->_processedFiles++;
-        if (!threadExecutor->_settings._errorsOnly) {
+        if (!threadExecutor->_settings.quiet) {
             EnterCriticalSection(&threadExecutor->_reportSync);
             CppCheckExecutor::reportStatus(threadExecutor->_processedFiles, threadExecutor->_totalFiles, threadExecutor->_processedSize, threadExecutor->_totalFileSize);
             LeaveCriticalSection(&threadExecutor->_reportSync);
         }
-
-        LeaveCriticalSection(&threadExecutor->_fileSync);
-    };
-
+    }
     return result;
 }
 
@@ -470,7 +488,7 @@ void ThreadExecutor::report(const ErrorLogger::ErrorMessage &msg, MessageType ms
 
     // Alert only about unique errors
     bool reportError = false;
-    std::string errmsg = msg.toString(_settings._verbose);
+    std::string errmsg = msg.toString(_settings.verbose);
 
     EnterCriticalSection(&_errorSync);
     if (std::find(_errorList.begin(), _errorList.end(), errmsg) == _errorList.end()) {
