@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ class ProjectFile : public QObject {
     Q_OBJECT
 
 public:
-    ProjectFile(QObject *parent = 0);
+    explicit ProjectFile(QObject *parent = 0);
     ProjectFile(const QString &filename, QObject *parent = 0);
 
     /**
