@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,23 +46,41 @@ ProjectFileDialog::ProjectFileDialog(const QString &path, QWidget *parent)
     // Checkboxes for the libraries..
     const QString applicationFilePath = QCoreApplication::applicationFilePath();
     const QString appPath = QFileInfo(applicationFilePath).canonicalPath();
-    const QString searchPaths[] = { appPath, appPath + "/cfg", inf.canonicalPath() };
-    for (int i = 0; i < 3; i++) {
-        QDir dir(searchPaths[i]);
+    QSettings settings;
+    const QString datadir = settings.value("DATADIR",QString()).toString();
+    QStringList searchPaths;
+    searchPaths << appPath << appPath + "/cfg" << inf.canonicalPath();
+    if (!datadir.isEmpty())
+        searchPaths << datadir << datadir + "/cfg";
+    QStringList libs;
+    foreach (const QString sp, searchPaths) {
+        QDir dir(sp);
         dir.setSorting(QDir::Name);
         dir.setNameFilters(QStringList("*.cfg"));
         dir.setFilter(QDir::Files | QDir::NoDotAndDotDot);
-        foreach(QFileInfo item, dir.entryInfoList()) {
+        foreach (QFileInfo item, dir.entryInfoList()) {
             QString library = item.fileName();
+            {
+                Library lib;
+                const QString fullfilename = sp + "/" + library;
+                const Library::Error err = lib.load(nullptr, fullfilename.toLatin1());
+                if (err.errorcode != Library::OK)
+                    continue;
+            }
             library.chop(4);
             if (library.compare("std", Qt::CaseInsensitive) == 0)
                 continue;
-            QCheckBox *checkbox = new QCheckBox(this);
-            checkbox->setText(library);
-            mUI.librariesLayout->addWidget(checkbox);
-            mLibraryCheckboxes << checkbox;
+            if (libs.indexOf(library) == -1)
+                libs << library;
         }
     }
+    qSort(libs);
+    foreach (const QString library, libs) {
+        QCheckBox *checkbox = new QCheckBox(this);
+        checkbox->setText(library);
+        mUI.librariesLayout->addWidget(checkbox);
+        mLibraryCheckboxes << checkbox;
+    }
 
     connect(mUI.mButtons, SIGNAL(accepted()), this, SLOT(accept()));
     connect(mUI.mBtnAddInclude, SIGNAL(clicked()), this, SLOT(AddIncludeDir()));
@@ -190,7 +208,7 @@ QStringList ProjectFileDialog::GetExcludedPaths() const
 QStringList ProjectFileDialog::GetLibraries() const
 {
     QStringList libraries;
-    foreach(const QCheckBox *checkbox, mLibraryCheckboxes) {
+    foreach (const QCheckBox *checkbox, mLibraryCheckboxes) {
         if (checkbox->isChecked())
             libraries << checkbox->text();
     }
@@ -216,7 +234,7 @@ void ProjectFileDialog::SetRootPath(const QString &root)
 
 void ProjectFileDialog::SetIncludepaths(const QStringList &includes)
 {
-    foreach(QString dir, includes) {
+    foreach (QString dir, includes) {
         AddIncludeDir(dir);
     }
 }
@@ -225,7 +243,7 @@ void ProjectFileDialog::SetDefines(const QStringList &defines)
 {
     QString definestr;
     QString define;
-    foreach(define, defines) {
+    foreach (define, defines) {
         definestr += define;
         definestr += ";";
     }
@@ -237,14 +255,14 @@ void ProjectFileDialog::SetDefines(const QStringList &defines)
 
 void ProjectFileDialog::SetPaths(const QStringList &paths)
 {
-    foreach(QString path, paths) {
+    foreach (QString path, paths) {
         AddPath(path);
     }
 }
 
 void ProjectFileDialog::SetExcludedPaths(const QStringList &paths)
 {
-    foreach(QString path, paths) {
+    foreach (QString path, paths) {
         AddExcludePath(path);
     }
 }
