@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,6 +34,8 @@
 #include <QFileInfo>
 #include <QFileDialog>
 #include <QClipboard>
+#include <QDesktopServices>
+#include <QUrl>
 #include <QContextMenuEvent>
 #include <QModelIndex>
 #include "common.h"
@@ -43,6 +45,8 @@
 #include "report.h"
 #include "application.h"
 #include "showtypes.h"
+#include "threadhandler.h"
+#include "path.h"
 
 ResultsTree::ResultsTree(QWidget * parent) :
     QTreeView(parent),
@@ -64,10 +68,19 @@ ResultsTree::~ResultsTree()
 {
 }
 
-void ResultsTree::Initialize(QSettings *settings, ApplicationList *list)
+void ResultsTree::keyPressEvent(QKeyEvent *event)
+{
+    if (event->key() == Qt::Key_Enter || event->key() == Qt::Key_Return) {
+        QuickStartApplication(this->currentIndex());
+    }
+    QTreeView::keyPressEvent(event);
+}
+
+void ResultsTree::Initialize(QSettings *settings, ApplicationList *list, ThreadHandler *checkThreadHandler)
 {
     mSettings = settings;
     mApplications = list;
+    mThread = checkThreadHandler;
     LoadSettings();
 }
 
@@ -80,6 +93,15 @@ QStandardItem *ResultsTree::CreateNormalItem(const QString &name)
     return item;
 }
 
+QStandardItem *ResultsTree::CreateCheckboxItem(bool checked)
+{
+    QStandardItem *item = new QStandardItem;
+    item->setCheckable(true);
+    item->setCheckState(checked ? Qt::Checked : Qt::Unchecked);
+    item->setEnabled(false);
+    return item;
+}
+
 QStandardItem *ResultsTree::CreateLineNumberItem(const QString &linenumber)
 {
     QStandardItem *item = new QStandardItem();
@@ -90,10 +112,10 @@ QStandardItem *ResultsTree::CreateLineNumberItem(const QString &linenumber)
     return item;
 }
 
-void ResultsTree::AddErrorItem(const ErrorItem &item)
+bool ResultsTree::AddErrorItem(const ErrorItem &item)
 {
     if (item.files.isEmpty()) {
-        return;
+        return false;
     }
 
     QString realfile = StripPath(item.files[0], false);
@@ -103,7 +125,6 @@ void ResultsTree::AddErrorItem(const ErrorItem &item)
     }
 
     bool hide = !mShowSeverities.isShown(item.severity);
-    //bool hide = !mShowTypes[SeverityToShowType(item.severity)];
 
     //If specified, filter on summary, message, filename, and id
     if (!hide && !mFilter.isEmpty()) {
@@ -130,13 +151,15 @@ void ResultsTree::AddErrorItem(const ErrorItem &item)
     line.severity = item.severity;
     //Create the base item for the error and ensure it has a proper
     //file item as a parent
-    QStandardItem *stditem = AddBacktraceFiles(EnsureFileItem(line.file, item.file0, hide),
+    QStandardItem* fileItem = EnsureFileItem(item.files[0], item.file0, hide);
+    QStandardItem* stditem = AddBacktraceFiles(fileItem,
                              line,
                              hide,
-                             SeverityToIcon(line.severity));
+                             SeverityToIcon(line.severity),
+                             false);
 
     if (!stditem)
-        return;
+        return false;
 
     //Add user data to that item
     QMap<QString, QVariant> data;
@@ -159,7 +182,8 @@ void ResultsTree::AddErrorItem(const ErrorItem &item)
         child_item = AddBacktraceFiles(stditem,
                                        line,
                                        hide,
-                                       ":images/go-down.png");
+                                       ":images/go-down.png",
+                                       true);
 
         //Add user data to that item
         QMap<QString, QVariant> child_data;
@@ -173,17 +197,18 @@ void ResultsTree::AddErrorItem(const ErrorItem &item)
         child_item->setData(QVariant(child_data));
     }
 
-    //TODO just hide/show current error and it's file
-    //since this does a lot of unnecessary work
+    // Partially refresh the tree: Unhide file item if necessary
     if (!hide) {
-        ShowFileItem(realfile);
+        setRowHidden(fileItem->row(), QModelIndex(), false);
     }
+    return true;
 }
 
 QStandardItem *ResultsTree::AddBacktraceFiles(QStandardItem *parent,
         const ErrorLine &item,
         const bool hide,
-        const QString &icon)
+        const QString &icon,
+        bool childOfMessage)
 
 {
     if (!parent) {
@@ -194,42 +219,40 @@ QStandardItem *ResultsTree::AddBacktraceFiles(QStandardItem *parent,
     // Ensure shown path is with native separators
     const QString file = QDir::toNativeSeparators(item.file);
     list << CreateNormalItem(file);
-    const QString severity = SeverityToTranslatedString(item.severity);
-    list << CreateNormalItem(severity);
+    if (childOfMessage)
+        list << CreateNormalItem("");
+    else {
+        const QString severity = SeverityToTranslatedString(item.severity);
+        list << CreateNormalItem(severity);
+    }
     list << CreateLineNumberItem(QString("%1").arg(item.line));
-    list << CreateNormalItem(item.errorId);
+    if (childOfMessage)
+        list << CreateNormalItem("");
+    else
+        list << CreateNormalItem(item.errorId);
+    if (childOfMessage)
+        list << CreateNormalItem("");
+    else
+        list << CreateCheckboxItem(item.inconclusive);
     //TODO message has parameter names so we'll need changes to the core
     //cppcheck so we can get proper translations
-    QString summary;
-    if (item.inconclusive) {
-        summary = tr("[Inconclusive]");
-        summary += " ";
-    }
-    summary += item.summary.toLatin1();
-    list << CreateNormalItem(summary);
+    if (childOfMessage)
+        list << CreateNormalItem("");
+    else
+        list << CreateNormalItem(item.summary.toLatin1());
 
     // Check for duplicate rows and don't add them if found
     for (int i = 0; i < parent->rowCount(); i++) {
-        // The first column is the file name and is always the same so
-        // we skip it in other platforms than Windows, where filename case
-        // is ignored. So in Windows we can get filenames "header.h" and
-        // "Header.h" and must compare them as identical filenames.
+        // The first column is the file name and is always the same
 
         // the third column is the line number so check it first
         if (parent->child(i, 2)->text() == list[2]->text()) {
             // the second column is the severity so check it next
             if (parent->child(i, 1)->text() == list[1]->text()) {
-                // the fourth column is the summary so check it last
-                if (parent->child(i, 3)->text() == list[3]->text()) {
-#if defined(_WIN32)
-                    const QString first = parent->child(i, 0)->text().toLower();
-                    const QString second = list[0]->text().toLower();
-                    if (first == second)
-                        return 0;
-#else
+                // the sixth column is the summary so check it last
+                if (parent->child(i, 5)->text() == list[5]->text()) {
                     // this row matches so don't add it
                     return 0;
-#endif // _WIN32
                 }
             }
         }
@@ -280,9 +303,16 @@ QString ResultsTree::SeverityToTranslatedString(Severity::SeverityType severity)
 
 QStandardItem *ResultsTree::FindFileItem(const QString &name) const
 {
-    QList<QStandardItem *> list = mModel.findItems(name);
-    if (list.size() > 0) {
-        return list[0];
+    // The first column contains the file name. In Windows we can get filenames
+    // "header.h" and "Header.h" and must compare them as identical.
+
+    for (int i = 0; i < mModel.rowCount(); i++) {
+#ifdef _WIN32
+        if (QString::compare(mModel.item(i, 0)->text(), name, Qt::CaseInsensitive) == 0)
+#else
+        if (mModel.item(i, 0)->text() == name)
+#endif
+            return mModel.item(i, 0);
     }
     return 0;
 }
@@ -310,6 +340,21 @@ void ResultsTree::Clear(const QString &filename)
     }
 }
 
+void ResultsTree::ClearRecheckFile(const QString &filename)
+{
+    for (int i = 0; i < mModel.rowCount(); ++i) {
+        const QStandardItem *item = mModel.item(i, 0);
+        if (!item)
+            continue;
+
+        QVariantMap data = item->data().toMap();
+        if (filename == data["file"].toString()) {
+            mModel.removeRow(i);
+            break;
+        }
+    }
+}
+
 
 void ResultsTree::LoadSettings()
 {
@@ -323,6 +368,7 @@ void ResultsTree::LoadSettings()
     mShowFullPath = mSettings->value(SETTINGS_SHOW_FULL_PATH, false).toBool();
 
     ShowIdColumn(mSettings->value(SETTINGS_SHOW_ERROR_ID, false).toBool());
+    ShowInconclusiveColumn(mSettings->value(SETTINGS_INCONCLUSIVE_ERRORS, false).toBool());
 }
 
 void ResultsTree::SaveSettings() const
@@ -470,14 +516,6 @@ QStandardItem *ResultsTree::EnsureFileItem(const QString &fullpath, const QStrin
     return item;
 }
 
-void ResultsTree::ShowFileItem(const QString &name)
-{
-    QStandardItem *item = FindFileItem(name);
-    if (item) {
-        setRowHidden(0, mModel.indexFromItem(item), false);
-    }
-}
-
 void ResultsTree::contextMenuEvent(QContextMenuEvent * e)
 {
     QModelIndex index = indexAt(e->pos());
@@ -532,48 +570,65 @@ void ResultsTree::contextMenuEvent(QContextMenuEvent * e)
             }
 
             //Create an action for the application
-            QAction *copyfilename   = new QAction(tr("Copy filename"), &menu);
-            QAction *copypath       = new QAction(tr("Copy full path"), &menu);
-            QAction *copymessage    = new QAction(tr("Copy message"), &menu);
-            QAction *copymessageid  = new QAction(tr("Copy message id"), &menu);
-            QAction *hide           = new QAction(tr("Hide"), &menu);
+            QAction *recheckSelectedFiles   = new QAction(tr("Recheck"), &menu);
+            QAction *copyfilename           = new QAction(tr("Copy filename"), &menu);
+            QAction *copypath               = new QAction(tr("Copy full path"), &menu);
+            QAction *copymessage            = new QAction(tr("Copy message"), &menu);
+            QAction *copymessageid          = new QAction(tr("Copy message id"), &menu);
+            QAction *hide                   = new QAction(tr("Hide"), &menu);
+            QAction *hideallid              = new QAction(tr("Hide all with id"), &menu);
+            QAction *opencontainingfolder   = new QAction(tr("Open containing folder"), &menu);
 
             if (multipleSelection) {
                 copyfilename->setDisabled(true);
                 copypath->setDisabled(true);
                 copymessage->setDisabled(true);
                 copymessageid->setDisabled(true);
+                hideallid->setDisabled(true);
+                opencontainingfolder->setDisabled(true);
             }
+            if (mThread->IsChecking())
+                recheckSelectedFiles->setDisabled(true);
+            else
+                recheckSelectedFiles->setDisabled(false);
 
+            menu.addAction(recheckSelectedFiles);
             menu.addAction(copyfilename);
             menu.addAction(copypath);
             menu.addAction(copymessage);
             menu.addAction(copymessageid);
             menu.addAction(hide);
+            menu.addAction(hideallid);
+            menu.addAction(opencontainingfolder);
 
+            connect(recheckSelectedFiles, SIGNAL(triggered()), this, SLOT(RecheckSelectedFiles()));
             connect(copyfilename, SIGNAL(triggered()), this, SLOT(CopyFilename()));
             connect(copypath, SIGNAL(triggered()), this, SLOT(CopyFullPath()));
             connect(copymessage, SIGNAL(triggered()), this, SLOT(CopyMessage()));
             connect(copymessageid, SIGNAL(triggered()), this, SLOT(CopyMessageId()));
             connect(hide, SIGNAL(triggered()), this, SLOT(HideResult()));
+            connect(hideallid, SIGNAL(triggered()), this, SLOT(HideAllIdResult()));
+            connect(opencontainingfolder, SIGNAL(triggered()), this, SLOT(OpenContainingFolder()));
         }
 
         //Start the menu
         menu.exec(e->globalPos());
+        index = indexAt(e->pos());
+        if (index.isValid()) {
+            mContextItem = mModel.itemFromIndex(index);
+            if (mContextItem && mApplications->GetApplicationCount() > 0 && mContextItem->parent()) {
+                //Disconnect all signals
+                for (int i = 0; i < actions.size(); i++) {
 
-        if (mContextItem && mApplications->GetApplicationCount() > 0 && mContextItem->parent()) {
-            //Disconnect all signals
-            for (int i = 0; i < actions.size(); i++) {
+                    disconnect(actions[i], SIGNAL(triggered()), signalMapper, SLOT(map()));
+                }
 
-                disconnect(actions[i], SIGNAL(triggered()), signalMapper, SLOT(map()));
+                disconnect(signalMapper, SIGNAL(mapped(int)),
+                           this, SLOT(Context(int)));
+                //And remove the signal mapper
+                delete signalMapper;
             }
-
-            disconnect(signalMapper, SIGNAL(mapped(int)),
-                       this, SLOT(Context(int)));
-            //And remove the signal mapper
-            delete signalMapper;
         }
-
     }
 }
 
@@ -616,7 +671,7 @@ void ResultsTree::StartApplication(QStandardItem *target, int application)
         //Replace (file) with filename
         QString file = data["file"].toString();
         file = QDir::toNativeSeparators(file);
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN
         file.replace(QString("\\"), QString("\\\\"));
 #endif
         qDebug() << "Opening file: " << file;
@@ -659,14 +714,14 @@ void ResultsTree::StartApplication(QStandardItem *target, int application)
         QString program = app.getPath();
 
         // In Windows we must surround paths including spaces with quotation marks.
-#ifdef Q_WS_WIN
+#ifdef Q_OS_WIN
         if (program.indexOf(" ") > -1) {
             if (!program.startsWith('"') && !program.endsWith('"')) {
                 program.insert(0, "\"");
                 program.append("\"");
             }
         }
-#endif // Q_WS_WIN
+#endif // Q_OS_WIN
 
         const QString cmdLine = QString("%1 %2").arg(program).arg(params);
 
@@ -694,20 +749,21 @@ QString ResultsTree::AskFileDir(const QString &file)
     msgbox.exec();
 
     QString dir = QFileDialog::getExistingDirectory(this, tr("Select Directory"),
-                  "",
+                  GetPath(SETTINGS_LAST_SOURCE_PATH),
                   QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
     mCheckPath = dir;
+    SetPath(SETTINGS_LAST_SOURCE_PATH, dir);
     return dir;
 }
 
 void ResultsTree::CopyFilename()
 {
-    CopyPath(mContextItem, false);
+    CopyPathToClipboard(mContextItem, false);
 }
 
 void ResultsTree::CopyFullPath()
 {
-    CopyPath(mContextItem, true);
+    CopyPathToClipboard(mContextItem, true);
 }
 
 void ResultsTree::CopyMessage()
@@ -752,8 +808,7 @@ void ResultsTree::HideResult()
         return;
 
     QModelIndexList selectedRows = mSelectionModel->selectedRows();
-    QModelIndex index;
-    foreach(index, selectedRows) {
+    foreach (QModelIndex index, selectedRows) {
         QStandardItem *item = mModel.itemFromIndex(index);
         //Set the "hide" flag for this item
         QVariantMap data = item->data().toMap();
@@ -765,6 +820,80 @@ void ResultsTree::HideResult()
     }
 }
 
+void ResultsTree::RecheckSelectedFiles()
+{
+    if (!mSelectionModel)
+        return;
+
+    QModelIndexList selectedRows = mSelectionModel->selectedRows();
+    QStringList selectedItems;
+    foreach (QModelIndex index, selectedRows) {
+        QStandardItem *item = mModel.itemFromIndex(index);
+        while (item->parent())
+            item = item->parent();
+        QVariantMap data = item->data().toMap();
+        QString currentFile = data["file"].toString();
+        if (Path::isHeader(currentFile.toStdString())) {
+            if (!selectedItems.contains(data["file0"].toString()))
+                selectedItems<<data["file0"].toString();
+        }
+        if (!selectedItems.contains(currentFile))
+            selectedItems<<currentFile;
+    }
+    emit CheckSelected(selectedItems);
+}
+
+void ResultsTree::HideAllIdResult()
+{
+    if (mContextItem && mContextItem->parent()) {
+        // Make sure we are working with the first column
+        if (mContextItem->column() != 0)
+            mContextItem = mContextItem->parent()->child(mContextItem->row(), 0);
+        QVariantMap data = mContextItem->data().toMap();
+
+        QString messageId = data["id"].toString();
+
+        // hide all errors with that message Id
+        int filecount = mModel.rowCount();
+        for (int i = 0; i < filecount; i++) {
+            //Get file i
+            QStandardItem *file = mModel.item(i, 0);
+            if (!file) {
+                continue;
+            }
+
+            //Get the amount of errors this file contains
+            int errorcount = file->rowCount();
+
+            for (int j = 0; j < errorcount; j++) {
+                //Get the error itself
+                QStandardItem *child = file->child(j, 0);
+                if (!child) {
+                    continue;
+                }
+
+                QVariantMap userdata = child->data().toMap();
+                if (userdata["id"].toString() == messageId) {
+                    userdata["hide"] = true;
+                    child->setData(QVariant(userdata));
+                }
+            }
+        }
+
+        RefreshTree();
+        emit ResultsHidden(true);
+    }
+}
+
+void ResultsTree::OpenContainingFolder()
+{
+    QString filePath = GetFilePath(mContextItem, true);
+    if (!filePath.isEmpty()) {
+        filePath = QFileInfo(filePath).absolutePath();
+        QDesktopServices::openUrl(QUrl::fromLocalFile(filePath));
+    }
+}
+
 void ResultsTree::Context(int application)
 {
     StartApplication(mContextItem, application);
@@ -775,7 +904,13 @@ void ResultsTree::QuickStartApplication(const QModelIndex &index)
     StartApplication(mModel.itemFromIndex(index));
 }
 
-void ResultsTree::CopyPath(QStandardItem *target, bool fullPath)
+void ResultsTree::CopyPathToClipboard(QStandardItem *target, bool fullPath)
+{
+    QClipboard *clipboard = QApplication::clipboard();
+    clipboard->setText(GetFilePath(target, fullPath));
+}
+
+QString ResultsTree::GetFilePath(QStandardItem *target, bool fullPath)
 {
     if (target) {
         // Make sure we are working with the first column
@@ -793,9 +928,10 @@ void ResultsTree::CopyPath(QStandardItem *target, bool fullPath)
             pathStr = fi.fileName();
         }
 
-        QClipboard *clipboard = QApplication::clipboard();
-        clipboard->setText(pathStr);
+        return pathStr;
     }
+
+    return QString();
 }
 
 QString ResultsTree::SeverityToIcon(Severity::SeverityType severity) const
@@ -824,7 +960,7 @@ void ResultsTree::SaveResults(Report *report) const
 
     for (int i = 0; i < mModel.rowCount(); i++) {
         QStandardItem *item = mModel.item(i, 0);
-        if (!isRowHidden(i, item->index()))
+        if (!isRowHidden(i, QModelIndex()))
             SaveErrors(report, item);
     }
 
@@ -887,7 +1023,8 @@ void ResultsTree::SaveErrors(Report *report, QStandardItem *item) const
 void ResultsTree::UpdateSettings(bool showFullPath,
                                  bool saveFullPath,
                                  bool saveAllErrors,
-                                 bool showErrorId)
+                                 bool showErrorId,
+                                 bool showInconclusive)
 {
     if (mShowFullPath != showFullPath) {
         mShowFullPath = showFullPath;
@@ -898,6 +1035,7 @@ void ResultsTree::UpdateSettings(bool showFullPath,
     mSaveAllErrors = saveAllErrors;
 
     ShowIdColumn(showErrorId);
+    ShowInconclusiveColumn(showInconclusive);
 }
 
 void ResultsTree::SetCheckDirectory(const QString &dir)
@@ -953,7 +1091,7 @@ void ResultsTree::RefreshFilePaths(QStandardItem *item)
                 if (!child) {
                     continue;
                 }
-                //Get childs's user data
+                //Get child's user data
                 QVariant child_userdata = child->data();
                 //Convert it to QVariantMap
                 QVariantMap child_data = child_userdata.toMap();
@@ -997,7 +1135,7 @@ bool ResultsTree::HasResults() const
 void ResultsTree::Translate()
 {
     QStringList labels;
-    labels << tr("File") << tr("Severity") << tr("Line") << tr("Id") << tr("Summary");
+    labels << tr("File") << tr("Severity") << tr("Line") << tr("Id") << tr("Inconclusive") << tr("Summary");
     mModel.setHorizontalHeaderLabels(labels);
     //TODO go through all the errors in the tree and translate severity and message
 }
@@ -1011,6 +1149,14 @@ void ResultsTree::ShowIdColumn(bool show)
         hideColumn(3);
 }
 
+void ResultsTree::ShowInconclusiveColumn(bool show)
+{
+    if (show)
+        showColumn(4);
+    else
+        hideColumn(4);
+}
+
 void ResultsTree::currentChanged(const QModelIndex &current, const QModelIndex &previous)
 {
     QTreeView::currentChanged(current, previous);
