@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -35,6 +35,7 @@ class ErrorLine;
 class QModelIndex;
 class QWidget;
 class QItemSelectionModel;
+class ThreadHandler;
 
 /// @addtogroup GUI
 /// @{
@@ -47,16 +48,16 @@ class QItemSelectionModel;
 class ResultsTree : public QTreeView {
     Q_OBJECT
 public:
-    ResultsTree(QWidget * parent = 0);
+    explicit ResultsTree(QWidget * parent = 0);
     virtual ~ResultsTree();
-    void Initialize(QSettings *settings, ApplicationList *list);
+    void Initialize(QSettings *settings, ApplicationList *list, ThreadHandler *checkThreadHandler);
 
     /**
     * @brief Add a new item to the tree
     *
     * @param item Error item data
     */
-    void AddErrorItem(const ErrorItem &item);
+    bool AddErrorItem(const ErrorItem &item);
 
     /**
     * @brief Clear all errors from the tree
@@ -69,6 +70,11 @@ public:
      */
     void Clear(const QString &filename);
 
+    /**
+     * @brief Clear errors of a file selected for recheck
+     */
+    void ClearRecheckFile(const QString &filename);
+
     /**
     * @brief Function to show/hide certain type of errors
     * Refreshes the tree.
@@ -104,8 +110,9 @@ public:
     * @param saveFullPath Save full path of files in reports
     * @param saveAllErrors Save all visible errors
     * @param showErrorId Show error id
+    * @param showInconclusive Show inconclusive column
     */
-    void UpdateSettings(bool showFullPath, bool saveFullPath, bool saveAllErrors, bool showErrorId);
+    void UpdateSettings(bool showFullPath, bool saveFullPath, bool saveAllErrors, bool showErrorId, bool showInconclusive);
 
     /**
     * @brief Set the directory we are checking
@@ -144,6 +151,11 @@ public:
     */
     void ShowIdColumn(bool show);
 
+    /**
+    * @brief Show optional column "Inconclusve"
+    */
+    void ShowInconclusiveColumn(bool show);
+
     /**
     * @brief Returns true if column "Id" is shown
     */
@@ -156,6 +168,7 @@ public:
      */
     ShowTypes mShowSeverities;
 
+    virtual void keyPressEvent(QKeyEvent *event);
 
 signals:
     /**
@@ -165,6 +178,13 @@ signals:
     */
     void ResultsHidden(bool hidden);
 
+    /**
+    * @brief Signal to perform selected files recheck
+    *
+    * @param selectedItems list of selected files
+    */
+    void CheckSelected(QStringList selectedItems);
+
     /**
     * @brief Signal for selection change in result tree.
     *
@@ -217,6 +237,23 @@ protected slots:
     */
     void HideResult();
 
+    /**
+    * @brief Slot for rechecking selected files
+    *
+    */
+    void RecheckSelectedFiles();
+
+    /**
+    * @brief Slot for context menu item to hide all messages with the current message Id
+    *
+    */
+    void HideAllIdResult();
+
+    /**
+    * @brief Slot for context menu item to open the folder containing the current file.
+    */
+    void OpenContainingFolder();
+
     /**
     * @brief Slot for selection change in the results tree.
     *
@@ -279,7 +316,15 @@ protected:
     * @param target Error tree item to open
     * @param fullPath Are we copying full path or only filename?
     */
-    void CopyPath(QStandardItem *target, bool fullPath);
+    void CopyPathToClipboard(QStandardItem *target, bool fullPath);
+
+    /**
+    * @brief Helper function returning the filename/full path of the error tree item \a target.
+    *
+    * @param target The error tree item containing the filename/full path
+    * @param fullPath Whether or not to retrieve the full path or only the filename.
+    */
+    QString GetFilePath(QStandardItem *target, bool fullPath);
 
     /**
     * @brief Context menu event (user right clicked on the tree)
@@ -295,12 +340,14 @@ protected:
     * @param item Error line data
     * @param hide Should this be hidden (true) or shown (false)
     * @param icon Should a default backtrace item icon be added
+    * @param childOfMessage Is this a child element of a message?
     * @return newly created QStandardItem *
     */
     QStandardItem *AddBacktraceFiles(QStandardItem *parent,
                                      const ErrorLine &item,
                                      const bool hide,
-                                     const QString &icon);
+                                     const QString &icon,
+                                     bool childOfMessage);
 
 
     /**
@@ -339,6 +386,15 @@ protected:
     */
     static QStandardItem *CreateNormalItem(const QString &name);
 
+    /**
+    * @brief Create new normal item.
+    *
+    * Normal item has left alignment and text set also as tooltip.
+    * @param checked checked
+    * @return new QStandardItem
+    */
+    static QStandardItem *CreateCheckboxItem(bool checked);
+
     /**
     * @brief Create new line number item.
     *
@@ -367,13 +423,6 @@ protected:
     */
     QStandardItem *EnsureFileItem(const QString &fullpath, const QString &file0, bool hide);
 
-    /**
-    * @brief Show a file item
-    *
-    * @param name Filename of the fileitem
-    */
-    void ShowFileItem(const QString &name);
-
     /**
     * @brief Item model for tree
     *
@@ -442,6 +491,7 @@ protected:
 
 private:
     QItemSelectionModel *mSelectionModel;
+    ThreadHandler *mThread;
 };
 /// @}
 #endif // RESULTSTREE_H
