@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@ ScratchPad::ScratchPad(MainWindow& mainWindow)
 void ScratchPad::CheckButtonClicked()
 {
     QString filename = mUI.lineEdit->text();
-    if (filename.size() == 0)
+    if (filename.isEmpty())
         filename = "test.cpp";
     mMainWindow.CheckCode(mUI.plainTextEdit->toPlainText(), filename);
 }
