@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2012 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@ class MainWindow;
 class ScratchPad : public QDialog {
     Q_OBJECT
 public:
-    ScratchPad(MainWindow& mainWindow);
+    explicit ScratchPad(MainWindow& mainWindow);
 
 private slots:
     /**
