@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,15 +21,24 @@
 #include <QDebug>
 #include <QLocale>
 #include <QMessageBox>
+#include <QSettings>
+#include <QFileInfo>
 #include "translationhandler.h"
 
+
 // Provide own translations for standard buttons. This (garbage) code is needed to enforce them to appear in .ts files even after "lupdate gui.pro"
 static void unused()
 {
-    QT_TRANSLATE_NOOP("QDialogButtonBox", "OK");
-    QT_TRANSLATE_NOOP("QDialogButtonBox", "Cancel");
-    QT_TRANSLATE_NOOP("QDialogButtonBox", "Close");
-    QT_TRANSLATE_NOOP("QDialogButtonBox", "Save");
+    // Qt4
+    Q_UNUSED(QT_TRANSLATE_NOOP("QDialogButtonBox", "OK"));
+    Q_UNUSED(QT_TRANSLATE_NOOP("QDialogButtonBox", "Cancel"));
+    Q_UNUSED(QT_TRANSLATE_NOOP("QDialogButtonBox", "Close"));
+    Q_UNUSED(QT_TRANSLATE_NOOP("QDialogButtonBox", "Save"));
+    // Qt5
+    Q_UNUSED(QT_TRANSLATE_NOOP("QPlatformTheme", "OK"));
+    Q_UNUSED(QT_TRANSLATE_NOOP("QPlatformTheme", "Cancel"));
+    Q_UNUSED(QT_TRANSLATE_NOOP("QPlatformTheme", "Close"));
+    Q_UNUSED(QT_TRANSLATE_NOOP("QPlatformTheme", "Save"));
 }
 
 TranslationHandler::TranslationHandler(QObject *parent) :
@@ -39,19 +48,19 @@ TranslationHandler::TranslationHandler(QObject *parent) :
 {
     // Add our available languages
     // Keep this list sorted
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Chinese (Simplified)"), "cppcheck_zh_CN");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Dutch"), "cppcheck_nl");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "English"), "cppcheck_en");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Finnish"), "cppcheck_fi");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "French"), "cppcheck_fr");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "German"), "cppcheck_de");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Italian"), "cppcheck_it");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Japanese"), "cppcheck_ja");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Korean"), "cppcheck_ko");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Russian"), "cppcheck_ru");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Serbian"), "cppcheck_sr");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Spanish"), "cppcheck_es");
-    AddTranslation(QT_TRANSLATE_NOOP("MainWindow", "Swedish"), "cppcheck_sv");
+    AddTranslation("Chinese (Simplified)", "cppcheck_zh_CN");
+    AddTranslation("Dutch", "cppcheck_nl");
+    AddTranslation("English", "cppcheck_en");
+    AddTranslation("Finnish", "cppcheck_fi");
+    AddTranslation("French", "cppcheck_fr");
+    AddTranslation("German", "cppcheck_de");
+    AddTranslation("Italian", "cppcheck_it");
+    AddTranslation("Japanese", "cppcheck_ja");
+    AddTranslation("Korean", "cppcheck_ko");
+    AddTranslation("Russian", "cppcheck_ru");
+    AddTranslation("Serbian", "cppcheck_sr");
+    AddTranslation("Spanish", "cppcheck_es");
+    AddTranslation("Swedish", "cppcheck_sv");
 }
 
 TranslationHandler::~TranslationHandler()
@@ -61,7 +70,7 @@ TranslationHandler::~TranslationHandler()
 const QStringList TranslationHandler::GetNames() const
 {
     QStringList names;
-    foreach(TranslationInfo translation, mTranslations) {
+    foreach (TranslationInfo translation, mTranslations) {
         names.append(translation.mName);
     }
     return names;
@@ -97,7 +106,23 @@ bool TranslationHandler::SetLanguage(const QString &code)
         mTranslator = new QTranslator(this);
 
     //Load the new language
-    QString translationFile = "lang/" + mTranslations[index].mFilename;
+    const QString appPath = QFileInfo(QCoreApplication::applicationFilePath()).canonicalPath();
+
+    QSettings settings;
+    QString datadir = settings.value("DATADIR").toString();
+    if (datadir.isEmpty())
+        datadir = appPath;
+
+    QString translationFile;
+    if (QFile::exists(datadir + "/lang/" + mTranslations[index].mFilename + ".qm"))
+        translationFile = datadir + "/lang/" + mTranslations[index].mFilename + ".qm";
+
+    else if (QFile::exists(datadir + "/" + mTranslations[index].mFilename + ".qm"))
+        translationFile = datadir + "/" + mTranslations[index].mFilename + ".qm";
+
+    else
+        translationFile = appPath + "/" + mTranslations[index].mFilename + ".qm";
+
     if (!mTranslator->load(translationFile) && !failure) {
         translationFile += ".qm";
         //If it failed, lets check if the default file exists
@@ -141,13 +166,8 @@ QString TranslationHandler::GetCurrentLanguage() const
 
 QString TranslationHandler::SuggestLanguage() const
 {
-    /*
-    Get language from system locale's name
-    QLocale::languageToString would return the languages full name and we
-    only want two-letter ISO 639 language code so we'll get it from
-    locale's name.
-    */
-    QString language = QLocale::system().name().left(2);
+    //Get language from system locale's name ie sv_SE or zh_CN
+    QString language = QLocale::system().name();
     //qDebug()<<"Your language is"<<language;
 
     //And see if we can find it from our list of language files
@@ -166,7 +186,8 @@ void TranslationHandler::AddTranslation(const char *name, const char *filename)
     TranslationInfo info;
     info.mName = name;
     info.mFilename = filename;
-    info.mCode = QString(filename).right(2);
+    int codeLength = QString(filename).length() - QString(filename).indexOf('_') - 1;
+    info.mCode = QString(filename).right(codeLength);
     mTranslations.append(info);
 }
 
@@ -177,6 +198,9 @@ int TranslationHandler::GetLanguageIndexByCode(const QString &code) const
         if (mTranslations[i].mCode == code) {
             index = i;
             break;
+        } else if (mTranslations[i].mCode == code.left(2)) {
+            index = i;
+            break;
         }
     }
     return index;
