@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@
 */
 class XmlReportV2 : public XmlReport {
 public:
-    XmlReportV2(const QString &filename);
+    explicit XmlReportV2(const QString &filename);
     virtual ~XmlReportV2();
 
     /**
