@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -30,18 +30,6 @@ namespace {
     Check64BitPortability instance;
 }
 
-/** Is given variable a pointer or array? */
-static bool isaddr(const Variable *var)
-{
-    return (var && (var->isPointer() || var->isArray()));
-}
-
-/** Is given variable an integer variable */
-static bool isint(const Variable *var)
-{
-    return (var && Token::Match(var->nameToken()->previous(), "int|long|DWORD %var% !!["));
-}
-
 void Check64BitPortability::pointerassignment()
 {
     if (!_settings->isEnabled("portability"))
@@ -53,7 +41,7 @@ void Check64BitPortability::pointerassignment()
     const std::size_t functions = symbolDatabase->functionScopes.size();
     for (std::size_t i = 0; i < functions; ++i) {
         const Scope * scope = symbolDatabase->functionScopes[i];
-        if (scope->function == 0 || !scope->function->hasBody) // We only look for functions with a body
+        if (scope->function == 0 || !scope->function->hasBody()) // We only look for functions with a body
             continue;
 
         bool retPointer = false;
@@ -65,33 +53,27 @@ void Check64BitPortability::pointerassignment()
             continue;
 
         for (const Token* tok = scope->classStart->next(); tok != scope->classEnd; tok = tok->next()) {
-            if (Token::Match(tok, "return %var%|%num% [;+]") && !Token::simpleMatch(tok, "return 0 ;")) {
-                enum { NO, INT, PTR, PTRDIFF } type = NO;
-                for (const Token *tok2 = tok->next(); tok2; tok2 = tok2->next()) {
-                    if ((type == NO || type == INT) && Token::Match(tok2, "%var% [+;]") && isaddr(tok2->variable()))
-                        type = PTR;
-                    else if (type == NO && (tok2->isNumber() || isint(tok2->variable())))
-                        type = INT;
-                    else if (type == PTR && Token::Match(tok2, "- %var%") && isaddr(tok2->next()->variable()))
-                        type = PTRDIFF;
-                    else if (Token::Match(tok2, "(")) {
-                        type = NO;
-                        break;
-                    } else if (tok2->str() == "(") {
-                        // TODO: handle parentheses
-                        type = NO;
-                        break;
-                    } else if (type == PTR && Token::simpleMatch(tok2, "."))
-                        type = NO; // Reset after pointer reference, see #4642
-                    else if (tok2->str() == ";")
-                        break;
-                }
-
-                if (retPointer && (type == INT || type == PTRDIFF))
-                    returnIntegerError(tok);
-                else if (!retPointer && type == PTR)
-                    returnPointerError(tok);
+            // skip nested functions
+            if (tok->str() == "{") {
+                if (tok->scope()->type == Scope::ScopeType::eFunction)
+                    tok = tok->link();
             }
+
+            if (tok->str() != "return")
+                continue;
+
+            if (!tok->astOperand1() || tok->astOperand1()->isNumber())
+                continue;
+
+            const ValueType * const returnType = tok->astOperand1()->valueType();
+            if (!returnType)
+                continue;
+
+            if (retPointer && returnType->pointer == 0U)
+                returnIntegerError(tok);
+
+            if (!retPointer && returnType->pointer >= 1U)
+                returnPointerError(tok);
         }
     }
 
@@ -99,23 +81,28 @@ void Check64BitPortability::pointerassignment()
     for (std::size_t i = 0; i < functions; ++i) {
         const Scope * scope = symbolDatabase->functionScopes[i];
         for (const Token *tok = scope->classStart; tok && tok != scope->classEnd; tok = tok->next()) {
-            if (Token::Match(tok, "[;{}] %var% = %var% [;+]")) {
+            if (tok->str() != "=")
+                continue;
 
-                const Variable *var1(tok->next()->variable());
-                const Variable *var2(tok->tokAt(3)->variable());
+            const ValueType *lhstype = tok->astOperand1() ? tok->astOperand1()->valueType() : nullptr;
+            const ValueType *rhstype = tok->astOperand2() ? tok->astOperand2()->valueType() : nullptr;
+            if (!lhstype || !rhstype)
+                continue;
 
-                if (isaddr(var1) && isint(var2) && tok->strAt(4) != "+")
-                    assignmentIntegerToAddressError(tok->next());
+            // Assign integer to pointer..
+            if (lhstype->pointer >= 1U &&
+                !tok->astOperand2()->isNumber() &&
+                rhstype->pointer == 0U &&
+                rhstype->originalTypeName.empty() &&
+                rhstype->type == ValueType::Type::INT)
+                assignmentIntegerToAddressError(tok);
 
-                else if (isint(var1) && isaddr(var2) && !tok->tokAt(3)->isPointerCompare()) {
-                    // assigning address => warning
-                    // some trivial addition => warning
-                    if (Token::Match(tok->tokAt(4), "+ %any% !!;"))
-                        continue;
-
-                    assignmentAddressToIntegerError(tok->next());
-                }
-            }
+            // Assign pointer to integer..
+            if (rhstype->pointer >= 1U &&
+                lhstype->pointer == 0U &&
+                lhstype->originalTypeName.empty() &&
+                lhstype->type == ValueType::Type::INT)
+                assignmentAddressToIntegerError(tok);
         }
     }
 }
