@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,8 +25,6 @@
 #include "check.h"
 #include "config.h"
 
-class Token;
-
 /// @addtogroup Checks
 /// @{
 
@@ -55,6 +53,7 @@ public:
         checkInternal.checkMissingPercentCharacter();
         checkInternal.checkUnknownPattern();
         checkInternal.checkRedundantNextPrevious();
+        checkInternal.checkExtraWhitespace();
     }
 
     /** @brief %Check if a simple pattern is used inside Token::Match or Token::findmatch */
@@ -72,20 +71,29 @@ public:
     /** @brief %Check for inefficient usage of Token::next(), Token::previous() and Token::tokAt() */
     void checkRedundantNextPrevious();
 
+    /** @brief %Check if there is whitespace at the beginning or at the end of a pattern */
+    void checkExtraWhitespace();
+
 private:
+    void multiComparePatternError(const Token *tok, const std::string &pattern, const std::string &funcname);
     void simplePatternError(const Token *tok, const std::string &pattern, const std::string &funcname);
     void complexPatternError(const Token *tok, const std::string &pattern, const std::string &funcname);
     void missingPercentCharacterError(const Token *tok, const std::string &pattern, const std::string &funcname);
     void unknownPatternError(const Token* tok, const std::string& pattern);
     void redundantNextPreviousError(const Token* tok, const std::string& func1, const std::string& func2);
+    void orInComplexPattern(const Token *tok, const std::string &pattern, const std::string &funcname);
+    void extraWhitespaceError(const Token *tok, const std::string &pattern, const std::string &funcname);
 
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckInternal c(0, settings, errorLogger);
+        c.multiComparePatternError(0, ";|%type%", "Match");
         c.simplePatternError(0, "class {", "Match");
         c.complexPatternError(0, "%type% ( )", "Match");
         c.missingPercentCharacterError(0, "%num", "Match");
         c.unknownPatternError(0, "%typ");
         c.redundantNextPreviousError(0, "previous", "next");
+        c.orInComplexPattern(0, "||", "Match");
+        c.extraWhitespaceError(0, "%str% ", "Match");
     }
 
     static std::string myName() {
