@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,9 +24,7 @@
 
 #include "config.h"
 #include "check.h"
-#include "settings.h"
 
-class Token;
 class Function;
 class Variable;
 
@@ -54,26 +52,26 @@ public:
         // Checks
         checkOther.warningOldStylePointerCast();
         checkOther.invalidPointerCast();
-        checkOther.checkUnsignedDivision();
         checkOther.checkCharVariable();
-        checkOther.strPlusChar();
         checkOther.checkRedundantAssignment();
         checkOther.checkRedundantAssignmentInSwitch();
         checkOther.checkSuspiciousCaseInSwitch();
-        checkOther.checkSelfAssignment();
-        checkOther.checkDuplicateIf();
         checkOther.checkDuplicateBranch();
         checkOther.checkDuplicateExpression();
         checkOther.checkUnreachableCode();
         checkOther.checkSuspiciousSemicolon();
         checkOther.checkVariableScope();
-        checkOther.clarifyCondition();   // not simplified because ifAssign
         checkOther.checkSignOfUnsignedVariable();  // don't ignore casts (#3574)
         checkOther.checkIncompleteArrayFill();
-        checkOther.checkSuspiciousStringCompare();
         checkOther.checkVarFuncNullUB();
         checkOther.checkNanInArithmeticExpression();
         checkOther.checkCommaSeparatedReturn();
+        checkOther.checkRedundantPointerOp();
+        checkOther.checkZeroDivision();
+        checkOther.checkNegativeBitwiseShift();
+        checkOther.checkInterlockedDecrement();
+        checkOther.checkUnusedLabel();
+        checkOther.checkEvaluationOrder();
     }
 
     /** @brief Run checks against the simplified token list */
@@ -81,45 +79,27 @@ public:
         CheckOther checkOther(tokenizer, settings, errorLogger);
 
         // Checks
-        checkOther.oppositeInnerCondition();
         checkOther.clarifyCalculation();
         checkOther.clarifyStatement();
         checkOther.checkConstantFunctionParameter();
         checkOther.checkIncompleteStatement();
         checkOther.checkCastIntToCharAndBack();
 
-        checkOther.invalidFunctionUsage();
-        checkOther.checkZeroDivision();
-        checkOther.checkZeroDivisionOrUselessCondition();
-        checkOther.checkMathFunctions();
-
-        checkOther.redundantGetAndSetUserId();
-        checkOther.checkIncorrectLogicOperator();
         checkOther.checkMisusedScopedObject();
         checkOther.checkMemsetZeroBytes();
-        checkOther.checkIncorrectStringCompare();
+        checkOther.checkMemsetInvalid2ndParam();
         checkOther.checkSwitchCaseFallThrough();
-        checkOther.checkAlwaysTrueOrFalseStringCompare();
-        checkOther.checkModuloAlwaysTrueFalse();
         checkOther.checkPipeParameterSize();
 
         checkOther.checkInvalidFree();
-        checkOther.checkDoubleFree();
         checkOther.checkRedundantCopy();
-        checkOther.checkNegativeBitwiseShift();
         checkOther.checkSuspiciousEqualityComparison();
         checkOther.checkComparisonFunctionIsAlwaysTrueOrFalse();
     }
 
-    /** To check the dead code in a program, which is inaccessible due to the counter-conditions check in nested-if statements **/
-    void oppositeInnerCondition();
-
     /** @brief Clarify calculation for ".. a * b ? .." */
     void clarifyCalculation();
 
-    /** @brief Suspicious condition (assignment+comparison) */
-    void clarifyCondition();
-
     /** @brief Suspicious statement like '*A++;' */
     void clarifyStatement();
 
@@ -129,19 +109,6 @@ public:
     /** @brief Check for pointer casts to a type with an incompatible binary data representation */
     void invalidPointerCast();
 
-    /**
-     * @brief Invalid function usage (invalid input value / overlapping data)
-     *
-     * %Check that given function parameters are valid according to the standard
-     * - wrong radix given for strtol/strtoul
-     * - overlapping data when using sprintf/snprintf
-     * - wrong input value according to library
-     */
-    void invalidFunctionUsage();
-
-    /** @brief %Check for unsigned division */
-    void checkUnsignedDivision();
-
     /** @brief %Check scope of variables */
     void checkVariableScope();
     static bool checkInnerScope(const Token *tok, const Variable* var, bool& used);
@@ -158,24 +125,12 @@ public:
     /** @brief Incomplete statement. A statement that only contains a constant or variable */
     void checkIncompleteStatement();
 
-    /** @brief str plus char (unusual pointer arithmetic) */
-    void strPlusChar();
-
     /** @brief %Check zero division*/
     void checkZeroDivision();
 
-    /** @brief %Check zero division / useless condition */
-    void checkZeroDivisionOrUselessCondition();
-
     /** @brief Check for NaN (not-a-number) in an arithmetic expression */
     void checkNanInArithmeticExpression();
 
-    /** @brief %Check for parameters given to math function that do not make sense*/
-    void checkMathFunctions();
-
-    /** @brief % Check for seteuid(geteuid()) or setuid(getuid())*/
-    void redundantGetAndSetUserId();
-
     /** @brief copying to memory or assigning to a variable twice */
     void checkRedundantAssignment();
 
@@ -191,26 +146,14 @@ public:
     /** @brief %Check for switch case fall through without comment */
     void checkSwitchCaseFallThrough();
 
-    /** @brief %Check for assigning a variable to itself*/
-    void checkSelfAssignment();
-
-    /** @brief %Check for testing for mutual exclusion over ||*/
-    void checkIncorrectLogicOperator();
-
     /** @brief %Check for objects that are destroyed immediately */
     void checkMisusedScopedObject();
 
     /** @brief %Check for filling zero bytes with memset() */
     void checkMemsetZeroBytes();
 
-    /** @brief %Check for using bad usage of strncmp and substr */
-    void checkIncorrectStringCompare();
-
-    /** @brief %Check for comparison of a string literal with a char* variable */
-    void checkSuspiciousStringCompare();
-
-    /** @brief %Check for suspicious code where multiple if have the same expression (e.g "if (a) { } else if (a) { }") */
-    void checkDuplicateIf();
+    /** @brief %Check for invalid 2nd parameter of memset() */
+    void checkMemsetInvalid2ndParam();
 
     /** @brief %Check for suspicious code where if and else branch are the same (e.g "if (a) b = true; else b = true;") */
     void checkDuplicateBranch();
@@ -218,12 +161,6 @@ public:
     /** @brief %Check for suspicious code with the same expression on both sides of operator (e.g "if (a && a)") */
     void checkDuplicateExpression();
 
-    /** @brief %Check for suspicious code that compares string literals for equality */
-    void checkAlwaysTrueOrFalseStringCompare();
-
-    /** @brief %Check for suspicious usage of modulo (e.g. "if(var % 4 == 4)") */
-    void checkModuloAlwaysTrueFalse();
-
     /** @brief %Check for code that gets never executed, such as duplicate break statements */
     void checkUnreachableCode();
 
@@ -237,14 +174,10 @@ public:
     void checkInvalidFree();
     void invalidFreeError(const Token *tok, bool inconclusive);
 
-    /** @brief %Check for double free or double close operations */
-    void checkDoubleFree();
-    void doubleFreeError(const Token *tok, const std::string &varname);
-
     /** @brief %Check for code creating redundant copies */
     void checkRedundantCopy();
 
-    /** @brief %Check for bitwise operation with negative right operand */
+    /** @brief %Check for bitwise shift with negative right operand */
     void checkNegativeBitwiseShift();
 
     /** @brief %Check for buffers that are filled incompletely with memset and similar functions */
@@ -260,37 +193,37 @@ public:
     void checkCastIntToCharAndBack();
 
     /** @brief %Check for using of comparison functions evaluating always to true or false. */
-    void checkComparisonFunctionIsAlwaysTrueOrFalse(void);
+    void checkComparisonFunctionIsAlwaysTrueOrFalse();
+
+    /** @brief %Check for redundant pointer operations */
+    void checkRedundantPointerOp();
+
+    /** @brief %Check for race condition with non-interlocked access after InterlockedDecrement() */
+    void checkInterlockedDecrement();
+
+    /** @brief %Check for unused labels */
+    void checkUnusedLabel();
+
+    /** @brief %Check for expression that depends on order of evaluation of side effects */
+    void checkEvaluationOrder();
 
 private:
-    bool isUnsigned(const Variable *var) const;
-    static bool isSigned(const Variable *var);
-
     // Error messages..
     void checkComparisonFunctionIsAlwaysTrueOrFalseError(const Token* tok, const std::string &strFunctionName, const std::string &varName, const bool result);
     void checkCastIntToCharAndBackError(const Token *tok, const std::string &strFunctionName);
     void checkPipeParameterSizeError(const Token *tok, const std::string &strVarName, const std::string &strDim);
-    void oppositeInnerConditionError(const Token *tok);
     void clarifyCalculationError(const Token *tok, const std::string &op);
-    void clarifyConditionError(const Token *tok, bool assign, bool boolop);
     void clarifyStatementError(const Token* tok);
-    void redundantGetAndSetUserIdError(const Token *tok);
     void cstyleCastError(const Token *tok);
     void invalidPointerCastError(const Token* tok, const std::string& from, const std::string& to, bool inconclusive);
-    void sprintfOverlappingDataError(const Token *tok, const std::string &varname);
-    void invalidFunctionArgError(const Token *tok, const std::string &functionName, int argnr, const std::string &validstr);
-    void invalidFunctionArgBoolError(const Token *tok, const std::string &functionName, int argnr);
-    void udivError(const Token *tok, bool inconclusive);
     void passedByValueError(const Token *tok, const std::string &parname);
     void constStatementError(const Token *tok, const std::string &type);
     void charArrayIndexError(const Token *tok);
     void charBitOpError(const Token *tok);
     void variableScopeError(const Token *tok, const std::string &varname);
-    void strPlusCharError(const Token *tok);
-    void zerodivError(const Token *tok);
-    void zerodivcondError(const Token *tokcond, const Token *tokdiv);
+    void zerodivError(const Token *tok, bool inconclusive);
+    void zerodivcondError(const Token *tokcond, const Token *tokdiv, bool inconclusive);
     void nanInArithmeticExpressionError(const Token *tok);
-    void mathfunctionCallError(const Token *tok, const unsigned int numParam = 1);
     void redundantAssignmentError(const Token *tok1, const Token* tok2, const std::string& var, bool inconclusive);
     void redundantAssignmentInSwitchError(const Token *tok1, const Token *tok2, const std::string &var);
     void redundantCopyError(const Token *tok1, const Token* tok2, const std::string& var);
@@ -300,18 +233,13 @@ private:
     void suspiciousCaseInSwitchError(const Token* tok, const std::string& operatorString);
     void suspiciousEqualityComparisonError(const Token* tok);
     void selfAssignmentError(const Token *tok, const std::string &varname);
-    void incorrectLogicOperatorError(const Token *tok, const std::string &condition, bool always);
-    void redundantConditionError(const Token *tok, const std::string &text);
     void misusedScopeObjectError(const Token *tok, const std::string &varname);
     void memsetZeroBytesError(const Token *tok, const std::string &varname);
-    void incorrectStringCompareError(const Token *tok, const std::string& func, const std::string &string);
-    void incorrectStringBooleanError(const Token *tok, const std::string& string);
-    void duplicateIfError(const Token *tok1, const Token *tok2);
+    void memsetFloatError(const Token *tok, const std::string &var_value);
+    void memsetValueOutOfRangeError(const Token *tok, const std::string &value);
     void duplicateBranchError(const Token *tok1, const Token *tok2);
     void duplicateExpressionError(const Token *tok1, const Token *tok2, const std::string &op);
-    void alwaysTrueFalseStringCompareError(const Token *tok, const std::string& str1, const std::string& str2);
-    void alwaysTrueStringVariableCompareError(const Token *tok, const std::string& str1, const std::string& str2);
-    void suspiciousStringCompareError(const Token* tok, const std::string& var);
+    void duplicateExpressionTernaryError(const Token *tok);
     void duplicateBreakError(const Token *tok, bool inconclusive);
     void unreachableCodeError(const Token* tok, bool inconclusive);
     void unsignedLessThanZeroError(const Token *tok, const std::string &varname, bool inconclusive);
@@ -319,30 +247,27 @@ private:
     void unsignedPositiveError(const Token *tok, const std::string &varname, bool inconclusive);
     void pointerPositiveError(const Token *tok, bool inconclusive);
     void SuspiciousSemicolonError(const Token *tok);
-    void doubleCloseDirError(const Token *tok, const std::string &varname);
-    void moduloAlwaysTrueFalseError(const Token* tok, const std::string& maxVal);
-    void negativeBitwiseShiftError(const Token *tok);
+    void negativeBitwiseShiftError(const Token *tok, int op);
     void redundantCopyError(const Token *tok, const std::string &varname);
     void incompleteArrayFillError(const Token* tok, const std::string& buffer, const std::string& function, bool boolean);
     void varFuncNullUBError(const Token *tok);
     void commaSeparatedReturnError(const Token *tok);
+    void redundantPointerOpError(const Token* tok, const std::string& varname, bool inconclusive);
+    void raceAfterInterlockedDecrementError(const Token* tok);
+    void unusedLabelError(const Token* tok);
+    void unknownEvaluationOrder(const Token* tok);
 
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
         CheckOther c(0, settings, errorLogger);
 
         // error
-        c.sprintfOverlappingDataError(0, "varname");
-        c.invalidFunctionArgError(0, "func_name", 1, "1-4");
-        c.invalidFunctionArgBoolError(0, "func_name", 1);
-        c.udivError(0, false);
-        c.zerodivError(0);
-        c.zerodivcondError(0,0);
-        c.mathfunctionCallError(0);
+        c.zerodivError(0, false);
+        c.zerodivcondError(0,0,false);
         c.misusedScopeObjectError(NULL, "varname");
-        c.doubleFreeError(0, "varname");
         c.invalidPointerCastError(0, "float", "double", false);
-        c.negativeBitwiseShiftError(0);
+        c.negativeBitwiseShiftError(0,1);
         c.checkPipeParameterSizeError(0, "varname", "dimension");
+        c.raceAfterInterlockedDecrementError(0);
 
         //performance
         c.redundantCopyError(0, "varname");
@@ -352,34 +277,26 @@ private:
         // style/warning
         c.checkComparisonFunctionIsAlwaysTrueOrFalseError(0,"isless","varName",false);
         c.checkCastIntToCharAndBackError(0,"func_name");
-        c.oppositeInnerConditionError(0);
         c.cstyleCastError(0);
         c.passedByValueError(0, "parametername");
         c.constStatementError(0, "type");
         c.charArrayIndexError(0);
         c.charBitOpError(0);
         c.variableScopeError(0, "varname");
-        c.strPlusCharError(0);
         c.redundantAssignmentInSwitchError(0, 0, "var");
         c.redundantCopyInSwitchError(0, 0, "var");
         c.switchCaseFallThrough(0);
         c.suspiciousCaseInSwitchError(0, "||");
         c.suspiciousEqualityComparisonError(0);
         c.selfAssignmentError(0, "varname");
-        c.incorrectLogicOperatorError(0, "foo > 3 && foo < 4", true);
-        c.redundantConditionError(0, "If x > 10 the condition x > 11 is always true.");
         c.memsetZeroBytesError(0, "varname");
+        c.memsetFloatError(0, "varname");
+        c.memsetValueOutOfRangeError(0, "varname");
         c.clarifyCalculationError(0, "+");
-        c.clarifyConditionError(0, true, false);
         c.clarifyStatementError(0);
-        c.incorrectStringCompareError(0, "substr", "\"Hello World\"");
-        c.suspiciousStringCompareError(0, "foo");
-        c.incorrectStringBooleanError(0, "\"Hello World\"");
-        c.duplicateIfError(0, 0);
         c.duplicateBranchError(0, 0);
         c.duplicateExpressionError(0, 0, "&&");
-        c.alwaysTrueFalseStringCompareError(0, "str1", "str2");
-        c.alwaysTrueStringVariableCompareError(0, "varname1", "varname2");
+        c.duplicateExpressionTernaryError(0);
         c.duplicateBreakError(0, false);
         c.unreachableCodeError(0, false);
         c.unsignedLessThanZeroError(0, "varname", false);
@@ -387,11 +304,13 @@ private:
         c.pointerLessThanZeroError(0, false);
         c.pointerPositiveError(0, false);
         c.SuspiciousSemicolonError(0);
-        c.moduloAlwaysTrueFalseError(0, "1");
         c.incompleteArrayFillError(0, "buffer", "memset", false);
         c.varFuncNullUBError(0);
         c.nanInArithmeticExpressionError(0);
         c.commaSeparatedReturnError(0);
+        c.redundantPointerOpError(0, "varname", false);
+        c.unusedLabelError(0);
+        c.unknownEvaluationOrder(0);
     }
 
     static std::string myName() {
@@ -402,62 +321,54 @@ private:
         return "Other checks\n"
 
                // error
-               "* Assigning bool value to pointer (converting bool value to address)\n"
-               "* division with zero\n"
-               "* scoped object destroyed immediately after construction\n"
-               "* assignment in an assert statement\n"
-               "* incorrect length arguments for 'substr' and 'strncmp'\n"
-               "* free() or delete of an invalid memory location\n"
-               "* double free() or double closedir()\n"
-               "* bitwise operation with negative right operand\n"
-               "* provide wrong dimensioned array to pipe() system command (--std=posix)\n"
-               "* cast the return values of getc(),fgetc() and getchar() to character and compare it to EOF\n"
-               "* invalid input values for functions\n"
+               "- division with zero\n"
+               "- scoped object destroyed immediately after construction\n"
+               "- assignment in an assert statement\n"
+               "- free() or delete of an invalid memory location\n"
+               "- bitwise operation with negative right operand\n"
+               "- provide wrong dimensioned array to pipe() system command (--std=posix)\n"
+               "- cast the return values of getc(),fgetc() and getchar() to character and compare it to EOF\n"
+               "- race condition with non-interlocked access after InterlockedDecrement() call\n"
+               "- expression 'x = x++;' depends on order of evaluation of side effects\n"
 
                // warning
-               "* either division by zero or useless condition\n"
+               "- either division by zero or useless condition\n"
+               "- memset() with a value out of range as the 2nd parameter\n"
 
-               //performance
-               "* redundant data copying for const variable\n"
-               "* subsequent assignment or copying to a variable or buffer\n"
+               // performance
+               "- redundant data copying for const variable\n"
+               "- subsequent assignment or copying to a variable or buffer\n"
+
+               // portability
+               "- memset() with a float as the 2nd parameter\n"
+               "- Passing NULL pointer to function with variable number of arguments leads to UB.\n"
 
                // style
-               "* Find dead code which is inaccessible due to the counter-conditions check in nested if statements\n"
-               "* C-style pointer cast in cpp file\n"
-               "* casting between incompatible pointer types\n"
-               "* redundant if\n"
-               "* [[CheckUnsignedDivision|unsigned division]]\n"
-               "* passing parameter by value\n"
-               "* [[IncompleteStatement|Incomplete statement]]\n"
-               "* [[charvar|check how signed char variables are used]]\n"
-               "* variable scope can be limited\n"
-               "* condition that is always true/false\n"
-               "* unusual pointer arithmetic. For example: \"abc\" + 'd'\n"
-               "* redundant assignment in a switch statement\n"
-               "* redundant pre/post operation in a switch statement\n"
-               "* redundant bitwise operation in a switch statement\n"
-               "* redundant strcpy in a switch statement\n"
-               "* assignment of a variable to itself\n"
-               "* Suspicious case labels in switch()\n"
-               "* Suspicious equality comparisons\n"
-               "* mutual exclusion over || always evaluating to true\n"
-               "* Comparison of values leading always to true or false\n"
-               "* Clarify calculation with parentheses\n"
-               "* suspicious condition (assignment+comparison)\n"
-               "* suspicious condition (runtime comparison of string literals)\n"
-               "* suspicious condition (string literals as boolean)\n"
-               "* suspicious comparison of a string literal with a char* variable\n"
-               "* duplicate break statement\n"
-               "* unreachable code\n"
-               "* testing if unsigned variable is negative\n"
-               "* testing is unsigned variable is positive\n"
-               "* Suspicious use of ; at the end of 'if/for/while' statement.\n"
-               "* Comparisons of modulo results that are always true/false.\n"
-               "* Array filled incompletely using memset/memcpy/memmove.\n"
-               "* redundant get and set function of user id (--std=posix).\n"
-               "* Passing NULL pointer to function with variable number of arguments leads to UB on some platforms.\n"
-               "* NaN (not a number) value used in arithmetic expression.\n"
-               "* comma in return statement (the comma can easily be misread as a semicolon).\n";
+               "- C-style pointer cast in C++ code\n"
+               "- casting between incompatible pointer types\n"
+               "- passing parameter by value\n"
+               "- [Incomplete statement](IncompleteStatement)\n"
+               "- [check how signed char variables are used](CharVar)\n"
+               "- variable scope can be limited\n"
+               "- unusual pointer arithmetic. For example: \"abc\" + 'd'\n"
+               "- redundant assignment, increment, or bitwise operation in a switch statement\n"
+               "- redundant strcpy in a switch statement\n"
+               "- Suspicious case labels in switch()\n"
+               "- assignment of a variable to itself\n"
+               "- Comparison of values leading always to true or false\n"
+               "- Clarify calculation with parentheses\n"
+               "- suspicious comparison of '\\0' with a char* variable\n"
+               "- duplicate break statement\n"
+               "- unreachable code\n"
+               "- testing if unsigned variable is negative/positive\n"
+               "- Suspicious use of ; at the end of 'if/for/while' statement.\n"
+               "- Array filled incompletely using memset/memcpy/memmove.\n"
+               "- NaN (not a number) value used in arithmetic expression.\n"
+               "- comma in return statement (the comma can easily be misread as a semicolon).\n"
+               "- prefer erfc, expm1 or log1p to avoid loss of precision.\n"
+               "- identical code in both branches of if/else or ternary operator.\n"
+               "- redundant pointer operation on pointer like &*some_ptr.\n"
+               "- find unused 'goto' labels.\n";
     }
 };
 /// @}
