@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,11 +21,15 @@
 #include "checkunusedfunctions.h"
 #include "tokenize.h"
 #include "token.h"
+#include "symboldatabase.h"
 #include <cctype>
 //---------------------------------------------------------------------------
 
 
 
+// Register this check class
+CheckUnusedFunctions CheckUnusedFunctions::instance;
+
 
 //---------------------------------------------------------------------------
 // FUNCTION USAGE - Check for unused functions etc
@@ -33,78 +37,55 @@
 
 void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char FileName[], const Settings *settings)
 {
+    const bool doMarkup = settings->library.markupFile(FileName);
+    const SymbolDatabase* symbolDatabase = tokenizer.getSymbolDatabase();
+
     // Function declarations..
-    for (const Token *tok = tokenizer.tokens(); tok; tok = tok->next()) {
-        if (tok->fileIndex() != 0)
+    for (std::size_t i = 0; i < symbolDatabase->functionScopes.size(); i++) {
+        const Scope* scope = symbolDatabase->functionScopes[i];
+        const Function* func = scope->function;
+        if (!func || !func->token || scope->classStart->fileIndex() != 0)
             continue;
 
-        // token contains a ':' => skip to next ; or {
-        if (tok->str().find(":") != std::string::npos) {
-            while (tok && tok->str().find_first_of(";{"))
-                tok = tok->next();
-            if (tok)
-                continue;
-            break;
+        // Don't warn about functions that are marked by __attribute__((constructor)) or __attribute__((destructor))
+        if (func->isAttributeConstructor() || func->isAttributeDestructor() || func->type != Function::eFunction)
+            continue;
+
+        // Don't care about templates
+        if (tokenizer.isCPP() && func->retDef->str() == "template")
+            continue;
+
+        FunctionUsage &usage = _functions[func->name()];
+
+        if (!usage.lineNumber)
+            usage.lineNumber = func->token->linenr();
+
+        // No filename set yet..
+        if (usage.filename.empty()) {
+            usage.filename = tokenizer.list.getSourceFilePath();
         }
-
-        // If this is a template function, skip it
-        if (tok->previous() && tok->previous()->str() == ">")
-            continue;
-
-        const Token *funcname = 0;
-
-        if (Token::Match(tok, "%type% %var% ("))
-            funcname = tok->next();
-        else if (Token::Match(tok, "%type% *|& %var% ("))
-            funcname = tok->tokAt(2);
-        else if (Token::Match(tok, "%type% :: %var% (") && !Token::Match(tok, tok->strAt(2).c_str()))
-            funcname = tok->tokAt(2);
-
-        // Don't assume throw as a function name: void foo() throw () {}
-        if (Token::Match(tok->previous(), ")|const") || funcname == 0)
-            continue;
-
-        // Don't warn about functions that are marked by __attribute__((constructor))
-        if (tok->isAttributeConstructor() || funcname->isAttributeConstructor())
-            continue;
-
-        tok = funcname->linkAt(1);
-
-        // Check that ") {" is found..
-        if (! Token::Match(tok, ") const| {") &&
-            ! Token::Match(tok, ") const| throw ( ) {"))
-            funcname = 0;
-
-        if (funcname) {
-            FunctionUsage &func = _functions[ funcname->str()];
-
-            if (!func.lineNumber)
-                func.lineNumber = funcname->linenr();
-
-            // No filename set yet..
-            if (func.filename.empty()) {
-                func.filename = tokenizer.getSourceFilePath();
-            }
-            // Multiple files => filename = "+"
-            else if (func.filename != tokenizer.getSourceFilePath()) {
-                //func.filename = "+";
-                func.usedOtherFile |= func.usedSameFile;
-            }
+        // Multiple files => filename = "+"
+        else if (usage.filename != tokenizer.list.getSourceFilePath()) {
+            //func.filename = "+";
+            usage.usedOtherFile |= usage.usedSameFile;
         }
     }
 
     // Function usage..
-    const Token *scopeEnd = NULL;
     for (const Token *tok = tokenizer.tokens(); tok; tok = tok->next()) {
 
         // parsing of library code to find called functions
         if (settings->library.isexecutableblock(FileName, tok->str())) {
             const Token * markupVarToken = tok->tokAt(settings->library.blockstartoffset(FileName));
-            int scope = 1;
+            int scope = 0;
+            bool start = true;
             // find all function calls in library code (starts with '(', not if or while etc)
-            while (scope) {
+            while (scope || start) {
                 if (markupVarToken->str() == settings->library.blockstart(FileName)) {
                     scope++;
+                    if (start) {
+                        start = false;
+                    }
                 } else if (markupVarToken->str() == settings->library.blockend(FileName))
                     scope--;
                 else if (!settings->library.iskeyword(FileName, markupVarToken->str())) {
@@ -112,7 +93,7 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
                         _functions[markupVarToken->str()].usedOtherFile = true;
                     else if (markupVarToken->next()->str() == "(") {
                         FunctionUsage &func = _functions[markupVarToken->str()];
-                        func.filename = tokenizer.getSourceFilePath();
+                        func.filename = tokenizer.list.getSourceFilePath();
                         if (func.filename.empty() || func.filename == "+")
                             func.usedOtherFile = true;
                         else
@@ -123,110 +104,101 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
             }
         }
 
-        if (!settings->library.markupFile(FileName) // only check source files
+        if (!doMarkup // only check source files
             && settings->library.isexporter(tok->str()) && tok->next() != 0) {
-            const Token * qPropToken = tok;
-            qPropToken = qPropToken->next();
-            while (qPropToken && qPropToken->str() != ")") {
-                if (settings->library.isexportedprefix(tok->str(), qPropToken->str())) {
-                    const Token* qNextPropToken = qPropToken->next();
-                    const std::string value = qNextPropToken->str();
+            const Token * propToken = tok->next();
+            while (propToken && propToken->str() != ")") {
+                if (settings->library.isexportedprefix(tok->str(), propToken->str())) {
+                    const Token* nextPropToken = propToken->next();
+                    const std::string& value = nextPropToken->str();
                     if (_functions.find(value) != _functions.end()) {
                         _functions[value].usedOtherFile = true;
                     }
                 }
-                if (settings->library.isexportedsuffix(tok->str(), qPropToken->str())) {
-                    const Token* qNextPropToken = qPropToken->previous();
-                    const std::string value = qNextPropToken->str();
+                if (settings->library.isexportedsuffix(tok->str(), propToken->str())) {
+                    const Token* prevPropToken = propToken->previous();
+                    const std::string& value = prevPropToken->str();
                     if (value != ")" && _functions.find(value) != _functions.end()) {
                         _functions[value].usedOtherFile = true;
                     }
                 }
-                qPropToken = qPropToken->next();
+                propToken = propToken->next();
             }
         }
 
-        if (settings->library.markupFile(FileName)
-            && settings->library.isimporter(FileName, tok->str()) && tok->next()) {
-            const Token * qPropToken = tok;
-            qPropToken = qPropToken->next();
-            if (qPropToken->next()) {
-                qPropToken = qPropToken->next();
-                while (qPropToken && qPropToken->str() != ")") {
-                    const std::string value = qPropToken->str();
+        if (doMarkup && settings->library.isimporter(FileName, tok->str()) && tok->next()) {
+            const Token * propToken = tok->next();
+            if (propToken->next()) {
+                propToken = propToken->next();
+                while (propToken && propToken->str() != ")") {
+                    const std::string& value = propToken->str();
                     if (!value.empty()) {
                         _functions[value].usedOtherFile = true;
                         break;
                     }
-                    qPropToken = qPropToken->next();
+                    propToken = propToken->next();
                 }
             }
         }
 
-        if (settings->library.isreflection(FileName, tok->str())) {
-            const int index = settings->library.reflectionArgument(FileName, tok->str());
-            if (index >= 0) {
-                const Token * funcToken = tok->tokAt(index);
-                if (funcToken) {
-                    std::string value = funcToken->str();
+        if (settings->library.isreflection(tok->str())) {
+            const int argIndex = settings->library.reflectionArgument(tok->str());
+            if (argIndex >= 0) {
+                const Token * funcToken = tok->next();
+                int index = 0;
+                std::string value;
+                while (funcToken) {
+                    if (funcToken->str()==",") {
+                        if (++index == argIndex)
+                            break;
+                        value = "";
+                    } else
+                        value += funcToken->str();
+                    funcToken = funcToken->next();
+                }
+                if (index == argIndex) {
                     value = value.substr(1, value.length() - 2);
                     _functions[value].usedOtherFile = true;
                 }
             }
         }
 
-        if (scopeEnd == NULL) {
-            if (!Token::Match(tok, ")|= const| {"))
-                continue;
-            scopeEnd = tok;
-            while (scopeEnd->str() != "{")
-                scopeEnd = scopeEnd->next();
-            scopeEnd = scopeEnd->link();
-        } else if (tok == scopeEnd) {
-            scopeEnd = NULL;
-            continue;
-        }
+        const Token *funcname = nullptr;
 
-
-        const Token *funcname = 0;
-
-        if (Token::Match(tok->next(), "%var% (")) {
+        if (tok->scope()->isExecutable() && Token::Match(tok, "%name% (")) {
+            funcname = tok;
+        } else if (tok->scope()->isExecutable() && Token::Match(tok, "%name% <") && Token::simpleMatch(tok->linkAt(1), "> (")) {
+            funcname = tok;
+        } else if (Token::Match(tok, "[;{}.,()[=+-/|!?:]")) {
             funcname = tok->next();
-        }
-
-        else if (Token::Match(tok->next(), "%var% <") && Token::simpleMatch(tok->linkAt(2), "> (")) {
-            funcname = tok->next();
-        }
-
-        else if (Token::Match(tok, "[;{}.,()[=+-/&|!?:] %var% [(),;:}]"))
-            funcname = tok->next();
-
-        else if (Token::Match(tok, "[=(,] &| %var% :: %var%")) {
-            funcname = tok->next();
-            if (funcname->str() == "&")
+            if (funcname && funcname->str() == "&")
                 funcname = funcname->next();
-            while (Token::Match(funcname,"%var% :: %var%"))
+            if (funcname && funcname->str() == "::")
+                funcname = funcname->next();
+            while (Token::Match(funcname, "%name% :: %name%"))
                 funcname = funcname->tokAt(2);
-            if (!Token::Match(funcname, "%var% [,);]"))
+
+            if (!Token::Match(funcname, "%name% [(),;]:}]"))
                 continue;
         }
 
-        else
+        if (!funcname)
             continue;
 
         // funcname ( => Assert that the end parentheses isn't followed by {
-        if (Token::Match(funcname, "%var% (|<")) {
+        if (Token::Match(funcname, "%name% (|<")) {
             const Token *ftok = funcname->next();
             if (ftok->str() == "<")
                 ftok = ftok->link();
             if (Token::Match(ftok->linkAt(1), ") const|throw|{"))
-                funcname = NULL;
+                funcname = nullptr;
         }
 
         if (funcname) {
             FunctionUsage &func = _functions[ funcname->str()];
+            const std::string& called_from_file = tokenizer.list.getSourceFilePath();
 
-            if (func.filename.empty() || func.filename == "+")
+            if (func.filename.empty() || func.filename == "+" || func.filename != called_from_file)
                 func.usedOtherFile = true;
             else
                 func.usedSameFile = true;
@@ -237,15 +209,14 @@ void CheckUnusedFunctions::parseTokens(const Tokenizer &tokenizer, const char Fi
 
 
 
-void CheckUnusedFunctions::check(ErrorLogger * const errorLogger)
+void CheckUnusedFunctions::check(ErrorLogger * const errorLogger, const Settings& settings)
 {
     for (std::map<std::string, FunctionUsage>::const_iterator it = _functions.begin(); it != _functions.end(); ++it) {
         const FunctionUsage &func = it->second;
         if (func.usedOtherFile || func.filename.empty())
             continue;
         if (it->first == "main" ||
-            it->first == "WinMain" ||
-            it->first == "_tmain" ||
+            (settings.isWindowsPlatform() && (it->first == "WinMain" || it->first == "_tmain")) ||
             it->first == "if" ||
             (it->first.compare(0, 8, "operator") == 0 && it->first.size() > 8 && !std::isalnum(it->first[8])))
             continue;
@@ -285,3 +256,17 @@ void CheckUnusedFunctions::unusedFunctionError(ErrorLogger * const errorLogger,
     else
         reportError(errmsg);
 }
+
+Check::FileInfo *CheckUnusedFunctions::getFileInfo(const Tokenizer *tokenizer, const Settings *settings) const
+{
+    if (settings->isEnabled("unusedFunction") && settings->jobs == 1)
+        instance.parseTokens(*tokenizer, tokenizer->list.getFiles().front().c_str(), settings);
+    return nullptr;
+
+}
+
+void CheckUnusedFunctions::analyseWholeProgram(const std::list<Check::FileInfo*> &fileInfo, const Settings& settings, ErrorLogger &errorLogger)
+{
+    (void)fileInfo;
+    check(&errorLogger, settings);
+}
