@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,12 +21,12 @@
 #define errorloggerH
 //---------------------------------------------------------------------------
 
-#include <list>
-#include <string>
-
 #include "config.h"
 #include "suppressions.h"
 
+#include <list>
+#include <string>
+
 class Token;
 class TokenList;
 
@@ -35,9 +35,11 @@ class TokenList;
 
 /** @brief Simple container to be thrown when internal error is detected. */
 struct InternalError {
-    InternalError(const Token *tok, const std::string &errorMsg);
+    enum Type {SYNTAX, INTERNAL};
+    InternalError(const Token *tok, const std::string &errorMsg, Type type = INTERNAL);
     const Token *token;
     std::string errorMessage;
+    std::string id;
 };
 
 /** @brief enum class for severity. Used when reporting errors. */
@@ -185,7 +187,7 @@ public:
             void setfile(const std::string &file);
 
             /**
-             * Returns the location as a string. Format: [file:line]
+             * @return the location as a string. Format: [file:line]
              */
             std::string stringify() const;
 
@@ -214,8 +216,9 @@ public:
          * @param verbose use verbose message
          * @param outputFormat Empty string to use default output format
          * or template to be used. E.g. "{file}:{line},{severity},{id},{message}"
+        * @return formatted string
          */
-        std::string toString(bool verbose, const std::string &outputFormat = "") const;
+        std::string toString(bool verbose, const std::string &outputFormat = emptyString) const;
 
         std::string serialize() const;
         bool deserialize(const std::string &data);
@@ -227,6 +230,7 @@ public:
         std::string file0;
 
         Severity::SeverityType _severity;
+        unsigned int _cwe;
         bool _inconclusive;
 
         /** set short and verbose messages */
@@ -252,6 +256,8 @@ public:
          */
         static void findAndReplace(std::string &source, const std::string &searchFor, const std::string &replaceWith);
 
+        static std::string fixInvalidChars(const std::string& raw);
+
         /** Short message */
         std::string _shortMessage;
 
@@ -305,6 +311,13 @@ public:
     void reportUnmatchedSuppressions(const std::list<Suppressions::SuppressionEntry> &unmatched);
 
     static std::string callStackToString(const std::list<ErrorLogger::ErrorMessage::FileLocation> &callStack);
+
+    /**
+     * Convert XML-sensitive characters into XML entities
+     * @param str The input string containing XML-sensitive characters
+     * @return The ouput string containing XML entities
+     */
+    static std::string toxml(const std::string &str);
 };
 
 /// @}
