@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,9 +32,41 @@
 
 class CPPCHECKLIB MathLib {
 public:
+    /** @brief value class */
+    class value {
+    private:
+        long long intValue;
+        double doubleValue;
+        enum { INT, LONG, LONGLONG, FLOAT } type;
+        bool isUnsigned;
+
+        void promote(const value &v);
+
+    public:
+        explicit value(const std::string &s);
+        std::string str() const;
+        bool isInt() const {
+            return type != FLOAT;
+        }
+        bool isFloat() const {
+            return type == FLOAT;
+        }
+
+        double getDoubleValue() const {
+            return isFloat() ? doubleValue : (double)intValue;
+        }
+
+        static value calc(char op, const value &v1, const value &v2);
+        int compare(const value &v) const;
+        value add(int v) const;
+    };
+
     typedef long long bigint;
+    typedef unsigned long long biguint;
 
     static bigint toLongNumber(const std::string & str);
+    static biguint toULongNumber(const std::string & str);
+
     template<class T> static std::string toString(T value) {
         std::ostringstream result;
         result << value;
@@ -44,17 +76,23 @@ public:
 
     static bool isInt(const std::string & str);
     static bool isFloat(const std::string &str);
+    static bool isDecimalFloat(const std::string &str);
     static bool isNegative(const std::string &str);
     static bool isPositive(const std::string &str);
-    static bool isHex(const std::string& str);
+    static bool isDec(const std::string & str);
+    static bool isFloatHex(const std::string& str);
+    static bool isIntHex(const std::string& str);
     static bool isOct(const std::string& str);
     static bool isBin(const std::string& str);
 
+    static bool isValidIntegerSuffix(std::string::const_iterator it, std::string::const_iterator end);
+
     static std::string add(const std::string & first, const std::string & second);
     static std::string subtract(const std::string & first, const std::string & second);
     static std::string multiply(const std::string & first, const std::string & second);
     static std::string divide(const std::string & first, const std::string & second);
     static std::string mod(const std::string & first, const std::string & second);
+    static std::string incdec(const std::string & var, const std::string & op);
     static std::string calculate(const std::string & first, const std::string & second, char action);
 
     static std::string sin(const std::string & tok);
@@ -74,8 +112,19 @@ public:
      * @return true if given character is octal digit.
      */
     static bool isOctalDigit(char c);
+    static MathLib::bigint characterLiteralToLongNumber(const std::string& str);
+
 };
 
+MathLib::value operator+(const MathLib::value &v1, const MathLib::value &v2);
+MathLib::value operator-(const MathLib::value &v1, const MathLib::value &v2);
+MathLib::value operator*(const MathLib::value &v1, const MathLib::value &v2);
+MathLib::value operator/(const MathLib::value &v1, const MathLib::value &v2);
+MathLib::value operator%(const MathLib::value &v1, const MathLib::value &v2);
+MathLib::value operator&(const MathLib::value &v1, const MathLib::value &v2);
+MathLib::value operator|(const MathLib::value &v1, const MathLib::value &v2);
+MathLib::value operator^(const MathLib::value &v1, const MathLib::value &v2);
+
 template<> CPPCHECKLIB std::string MathLib::toString(double value); // Declare specialization to avoid linker problems
 
 /// @}
