@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,8 @@
 #include <list>
 #include <vector>
 #include <set>
+#include <algorithm>
+#include <map>
 
 #include "config.h"
 #include "token.h"
@@ -33,6 +35,7 @@
 class Tokenizer;
 class Settings;
 class ErrorLogger;
+class Library;
 
 class Scope;
 class SymbolDatabase;
@@ -46,7 +49,7 @@ enum AccessControl { Public, Protected, Private, Global, Namespace, Argument, Lo
  * @brief Array dimension information.
  */
 struct Dimension {
-    Dimension() : start(NULL), end(NULL), num(0), known(true) { }
+    Dimension() : start(nullptr), end(nullptr), num(0), known(true) { }
 
     const Token *start;  // size start token
     const Token *end;    // size end token
@@ -64,15 +67,28 @@ public:
         Unknown, True, False
     } needInitialization;
 
-    struct BaseInfo {
+    class BaseInfo {
+    public:
+        BaseInfo() :
+            type(nullptr), nameTok(nullptr), access(Public), isVirtual(false) {
+        }
+
         std::string name;
         const Type* type;
         const Token* nameTok;
         AccessControl access;  // public/protected/private
         bool isVirtual;
+        // allow ordering within containers
+        bool operator<(const BaseInfo& rhs) const {
+            return this->type < rhs.type;
+        }
     };
 
     struct FriendInfo {
+        FriendInfo() :
+            nameStart(nullptr), nameEnd(nullptr), type(nullptr) {
+        }
+
         const Token* nameStart;
         const Token* nameEnd;
         std::string name;
@@ -90,13 +106,29 @@ public:
     }
 
     const std::string& name() const {
-        static const std::string empty;
-        return classDef->next()->isName() ? classDef->strAt(1) : empty;
+        const Token* next = classDef->next();
+        if (next->isName())
+            return next->str();
+        return emptyString;
     }
 
     const Token *initBaseInfo(const Token *tok, const Token *tok1);
 
     const Function* getFunction(const std::string& funcName) const;
+
+    /**
+    * Check for circulare dependencies, i.e. loops within the class hierarchie
+    * @param ancestors list of ancestors. For internal usage only, clients should not supply this argument.
+    * @return true if there is a circular dependency
+    */
+    bool hasCircularDependencies(std::set<BaseInfo>* ancestors = nullptr) const;
+
+    /**
+    * Check for dependency
+    * @param ancestor potential ancestor
+    * @return true if there is a dependency
+    */
+    bool findDependency(const Type* ancestor) const;
 };
 
 /** @brief Information about a member variable. */
@@ -112,7 +144,11 @@ class CPPCHECKLIB Variable {
         fIsPointer   = (1 << 6), /** @brief pointer variable */
         fIsReference = (1 << 7), /** @brief reference variable */
         fIsRValueRef = (1 << 8), /** @brief rvalue reference variable */
-        fHasDefault  = (1 << 9)  /** @brief function argument with default value */
+        fHasDefault  = (1 << 9), /** @brief function argument with default value */
+        fIsStlType   = (1 << 10), /** @brief STL type ('std::') */
+        fIsStlString = (1 << 11), /** @brief std::string|wstring|basic_string&lt;T&gt;|u16string|u32string */
+        fIsIntType   = (1 << 12), /** @brief Integral type */
+        fIsFloatType = (1 << 13)  /** @brief Floating point type */
     };
 
     /**
@@ -120,7 +156,7 @@ class CPPCHECKLIB Variable {
      * @param flag_ flag to get state of
      * @return true if flag set or false in flag not set
      */
-    bool getFlag(int flag_) const {
+    bool getFlag(unsigned int flag_) const {
         return bool((_flags & flag_) != 0);
     }
 
@@ -129,22 +165,21 @@ class CPPCHECKLIB Variable {
      * @param flag_ flag to set state
      * @param state_ new state of flag
      */
-    void setFlag(int flag_, bool state_) {
+    void setFlag(unsigned int flag_, bool state_) {
         _flags = state_ ? _flags | flag_ : _flags & ~flag_;
     }
 
     /**
      * @brief parse and save array dimension information
-     * @param dimensions array dimensions vector
-     * @param tok the first '[' token of array declaration
+     * @param lib Library instance
      * @return true if array, false if not
      */
-    static bool arrayDimensions(std::vector<Dimension> &dimensions, const Token *tok);
+    bool arrayDimensions(const Library* lib);
 
 public:
     Variable(const Token *name_, const Token *start_, const Token *end_,
              std::size_t index_, AccessControl access_, const Type *type_,
-             const Scope *scope_)
+             const Scope *scope_, const Library* lib)
         : _name(name_),
           _start(start_),
           _end(end_),
@@ -153,7 +188,7 @@ public:
           _flags(0),
           _type(type_),
           _scope(scope_) {
-        evaluate();
+        evaluate(lib);
     }
 
     /**
@@ -188,18 +223,25 @@ public:
         return _end;
     }
 
+    /**
+     * Get end token of variable declaration
+     * E.g.
+     * int i[2][3] = ...
+     *   end token ^
+     * @return variable declaration end token
+     */
+    const Token *declEndToken() const;
+
     /**
      * Get name string.
      * @return name string
      */
     const std::string &name() const {
-        static const std::string noname;
-
         // name may not exist for function arguments
         if (_name)
             return _name->str();
 
-        return noname;
+        return emptyString;
     }
 
     /**
@@ -331,7 +373,7 @@ public:
      * @return true if array, false if not
      */
     bool isArray() const {
-        return getFlag(fIsArray);
+        return getFlag(fIsArray) && !getFlag(fIsPointer);
     }
 
     /**
@@ -342,6 +384,20 @@ public:
         return getFlag(fIsPointer);
     }
 
+    /**
+     * Is variable a pointer to an array
+     * @return true if pointer to array, false otherwise
+     */
+    bool isPointerToArray() const {
+        return isPointer() && getFlag(fIsArray);
+    }
+
+    /**
+     * Is variable an array of pointers
+     * @return true if array or pointers, false otherwise
+     */
+    bool isPointerArray() const;
+
     /**
      * Is array or pointer variable.
      * @return true if pointer or array, false otherwise
@@ -422,6 +478,75 @@ public:
         return _dimensions[index_].known;
     }
 
+    /**
+    * Checks if the variable is an STL type ('std::')
+    * E.g.:
+    *   std::string s;
+    *   ...
+    *   sVar->isStlType() == true
+    * @return true if it is an stl type and its type matches any of the types in 'stlTypes'
+    */
+    bool isStlType() const {
+        return getFlag(fIsStlType);
+    }
+
+    /**
+     * Checks if the variable is an STL type ('std::')
+     * E.g.:
+     *   std::string s;
+     *   ...
+     *   sVar->isStlType() == true
+     * @return true if it is an stl type and its type matches any of the types in 'stlTypes'
+     */
+    bool isStlStringType() const {
+        return getFlag(fIsStlString);
+    }
+
+    /**
+     * Checks if the variable is of any of the STL types passed as arguments ('std::')
+     * E.g.:
+     *   std::string s;
+     *   ...
+     *   const char *str[] = {"string", "wstring"};
+     *   sVar->isStlType(str) == true
+     * @param stlType stl type
+     * @return true if it is an stl type and its type matches any of the types in 'stlTypes'
+     */
+    bool isStlType(const std::string& stlType) const {
+        return isStlType() && stlType==_start->strAt(2);
+    }
+
+    /**
+     * Checks if the variable is of any of the STL types passed as arguments ('std::')
+     * E.g.:
+     *   std::string s;
+     *   ...
+     *   const std::set<std::string> str = make_container< std::set<std::string> >() << "string" << "wstring";
+     *   sVar->isStlType(str) == true
+     * @param stlTypes set of stl types
+     * @return true if it is an stl type and its type matches any of the types in 'stlTypes'
+     */
+    bool isStlType(const std::set<std::string>& stlTypes) const {
+        return isStlType() && stlTypes.find(_start->strAt(2))!=stlTypes.end();
+    }
+
+    /**
+    * Determine whether it's a floating number type
+    * @return true if the type is known and it's a floating type (float, double and long double) or a pointer/array to it
+    */
+    bool isFloatingType() const {
+        return getFlag(fIsFloatType);
+    }
+
+    /**
+     * Determine whether it's an integral number type
+     * @return true if the type is known and it's an integral type (bool, char, short, int, long long and their unsigned counter parts) or a pointer/array to it
+     */
+    bool isIntegralType() const {
+        return getFlag(fIsIntType);
+    }
+
+
 private:
     // only symbol database can change the type
     friend class SymbolDatabase;
@@ -450,7 +575,7 @@ private:
     AccessControl _access;  // public/protected/private
 
     /** @brief flags */
-    int _flags;
+    unsigned int _flags;
 
     /** @brief pointer to user defined type info (for known types) */
     const Type *_type;
@@ -462,37 +587,67 @@ private:
     std::vector<Dimension> _dimensions;
 
     /** @brief fill in information, depending on Tokens given at instantiation */
-    void evaluate();
+    void evaluate(const Library* lib);
 };
 
 class CPPCHECKLIB Function {
+    /** @brief flags mask used to access specific bit. */
+    enum {
+        fHasBody        = (1 << 0),  /** @brief has implementation */
+        fIsInline       = (1 << 1),  /** @brief implementation in class definition */
+        fIsConst        = (1 << 2),  /** @brief is const */
+        fIsVirtual      = (1 << 3),  /** @brief is virtual */
+        fIsPure         = (1 << 4),  /** @brief is pure virtual */
+        fIsStatic       = (1 << 5),  /** @brief is static */
+        fIsStaticLocal  = (1 << 6),  /** @brief is static local */
+        fIsExtern       = (1 << 7),  /** @brief is extern */
+        fIsFriend       = (1 << 8),  /** @brief is friend */
+        fIsExplicit     = (1 << 9),  /** @brief is explicit */
+        fIsDefault      = (1 << 10), /** @brief is default */
+        fIsDelete       = (1 << 11), /** @brief is delete */
+        fIsNoExcept     = (1 << 12), /** @brief is noexcept */
+        fIsThrow        = (1 << 13), /** @brief is throw */
+        fIsOperator     = (1 << 14), /** @brief is operator */
+        fHasLvalRefQual = (1 << 15), /** @brief has & lvalue ref-qualifier */
+        fHasRvalRefQual = (1 << 16)  /** @brief has && rvalue ref-qualifier */
+    };
+
+    /**
+     * Get specified flag state.
+     * @param flag flag to get state of
+     * @return true if flag set or false in flag not set
+     */
+    bool getFlag(unsigned int flag) const {
+        return bool((flags & flag) != 0);
+    }
+
+    /**
+     * Set specified flag state.
+     * @param flag flag to set state
+     * @param state new state of flag
+     */
+    void setFlag(unsigned int flag, bool state) {
+        flags = state ? flags | flag : flags & ~flag;
+    }
+
 public:
     enum Type { eConstructor, eCopyConstructor, eMoveConstructor, eOperatorEqual, eDestructor, eFunction };
 
     Function()
-        : tokenDef(NULL),
-          argDef(NULL),
-          token(NULL),
-          arg(NULL),
-          retDef(NULL),
-          retType(NULL),
-          functionScope(NULL),
-          nestedIn(NULL),
+        : tokenDef(nullptr),
+          argDef(nullptr),
+          token(nullptr),
+          arg(nullptr),
+          retDef(nullptr),
+          retType(nullptr),
+          functionScope(nullptr),
+          nestedIn(nullptr),
           initArgCount(0),
           type(eFunction),
           access(Public),
-          hasBody(false),
-          isInline(false),
-          isConst(false),
-          isVirtual(false),
-          isPure(false),
-          isStatic(false),
-          isFriend(false),
-          isExplicit(false),
-          isDefault(false),
-          isDelete(false),
-          isOperator(false),
-          retFuncPtr(false) {
+          noexceptArg(nullptr),
+          throwArg(nullptr),
+          flags(0) {
     }
 
     const std::string &name() const {
@@ -522,6 +677,128 @@ public:
     bool isDestructor() const {
         return type==eDestructor;
     }
+    bool isAttributeConstructor() const {
+        return tokenDef->isAttributeConstructor();
+    }
+    bool isAttributeDestructor() const {
+        return tokenDef->isAttributeDestructor();
+    }
+    bool isAttributePure() const {
+        return tokenDef->isAttributePure();
+    }
+    bool isAttributeConst() const {
+        return tokenDef->isAttributeConst();
+    }
+    bool isAttributeNoreturn() const {
+        return tokenDef->isAttributeNoreturn();
+    }
+    bool isAttributeNothrow() const {
+        return tokenDef->isAttributeNothrow();
+    }
+
+    bool hasBody() const {
+        return getFlag(fHasBody);
+    }
+    bool isInline() const {
+        return getFlag(fIsInline);
+    }
+    bool isConst() const {
+        return getFlag(fIsConst);
+    }
+    bool isVirtual() const {
+        return getFlag(fIsVirtual);
+    }
+    bool isPure() const {
+        return getFlag(fIsPure);
+    }
+    bool isStatic() const {
+        return getFlag(fIsStatic);
+    }
+    bool isStaticLocal() const {
+        return getFlag(fIsStaticLocal);
+    }
+    bool isExtern() const {
+        return getFlag(fIsExtern);
+    }
+    bool isFriend() const {
+        return getFlag(fIsFriend);
+    }
+    bool isExplicit() const {
+        return getFlag(fIsExplicit);
+    }
+    bool isDefault() const {
+        return getFlag(fIsDefault);
+    }
+    bool isDelete() const {
+        return getFlag(fIsDelete);
+    }
+    bool isNoExcept() const {
+        return getFlag(fIsNoExcept);
+    }
+    bool isThrow() const {
+        return getFlag(fIsThrow);
+    }
+    bool isOperator() const {
+        return getFlag(fIsOperator);
+    }
+    bool hasLvalRefQualifier() const {
+        return getFlag(fHasLvalRefQual);
+    }
+    bool hasRvalRefQualifier() const {
+        return getFlag(fHasRvalRefQual);
+    }
+
+    void hasBody(bool state) {
+        setFlag(fHasBody, state);
+    }
+    void isInline(bool state) {
+        setFlag(fIsInline, state);
+    }
+    void isConst(bool state) {
+        setFlag(fIsConst, state);
+    }
+    void isVirtual(bool state) {
+        setFlag(fIsVirtual, state);
+    }
+    void isPure(bool state) {
+        setFlag(fIsPure, state);
+    }
+    void isStatic(bool state) {
+        setFlag(fIsStatic, state);
+    }
+    void isStaticLocal(bool state) {
+        setFlag(fIsStaticLocal, state);
+    }
+    void isExtern(bool state) {
+        setFlag(fIsExtern, state);
+    }
+    void isFriend(bool state) {
+        setFlag(fIsFriend, state);
+    }
+    void isExplicit(bool state) {
+        setFlag(fIsExplicit, state);
+    }
+    void isDefault(bool state) {
+        setFlag(fIsDefault, state);
+    }
+    void isDelete(bool state) {
+        setFlag(fIsDelete, state);
+    }
+    void isNoExcept(bool state) {
+        setFlag(fIsNoExcept, state);
+    }
+    void isThrow(bool state) {
+        setFlag(fIsThrow, state);
+    }
+    void isOperator(bool state) {
+        setFlag(fIsOperator, state);
+    }
+    void hasLvalRefQualifier(bool state) {
+        setFlag(fHasLvalRefQual, state);
+    }
+    void hasRvalRefQualifier(bool state) {
+        setFlag(fHasRvalRefQual, state);
+    }
 
     const Token *tokenDef; // function name token in class definition
     const Token *argDef;   // function argument start '(' in class definition
@@ -535,23 +812,15 @@ public:
     unsigned int initArgCount; // number of args with default values
     Type type;             // constructor, destructor, ...
     AccessControl access;  // public/protected/private
-    bool hasBody;          // has implementation
-    bool isInline;         // implementation in class definition
-    bool isConst;          // is const
-    bool isVirtual;        // is virtual
-    bool isPure;           // is pure virtual
-    bool isStatic;         // is static
-    bool isFriend;         // is friend
-    bool isExplicit;       // is explicit
-    bool isDefault;        // is default
-    bool isDelete;         // is delete
-    bool isOperator;       // is operator
-    bool retFuncPtr;       // returns function pointer
+    const Token *noexceptArg;
+    const Token *throwArg;
 
     static bool argsMatch(const Scope *info, const Token *first, const Token *second, const std::string &path, unsigned int depth);
 
 private:
-    bool isImplicitlyVirtual_rec(const ::Type* type, bool& safe) const;
+    bool isImplicitlyVirtual_rec(const ::Type* baseType, bool& safe) const;
+
+    unsigned int flags;
 };
 
 class CPPCHECKLIB Scope {
@@ -564,7 +833,7 @@ public:
         const Scope *scope;
     };
 
-    enum ScopeType { eGlobal, eClass, eStruct, eUnion, eNamespace, eFunction, eIf, eElse, eElseIf, eFor, eWhile, eDo, eSwitch, eUnconditional, eTry, eCatch };
+    enum ScopeType { eGlobal, eClass, eStruct, eUnion, eNamespace, eFunction, eIf, eElse, eFor, eWhile, eDo, eSwitch, eUnconditional, eTry, eCatch, eLambda };
 
     Scope(const SymbolDatabase *check_, const Token *classDef_, const Scope *nestedIn_);
     Scope(const SymbolDatabase *check_, const Token *classDef_, const Scope *nestedIn_, ScopeType type_, const Token *start_);
@@ -575,6 +844,7 @@ public:
     const Token *classStart; // '{' token
     const Token *classEnd;   // '}' token
     std::list<Function> functionList;
+    std::multimap<std::string, const Function *> functionMap;
     std::list<Variable> varlist;
     const Scope *nestedIn;
     std::list<Scope *> nestedList;
@@ -598,7 +868,7 @@ public:
     }
 
     bool isLocal() const {
-        return (type == eIf || type == eElse || type == eElseIf ||
+        return (type == eIf || type == eElse ||
                 type == eFor || type == eWhile || type == eDo ||
                 type == eSwitch || type == eUnconditional ||
                 type == eTry || type == eCatch);
@@ -607,9 +877,10 @@ public:
     /**
      * @brief find a function
      * @param tok token of function call
+     * @param requireConst if const refers to a const variable only const methods should be matched
      * @return pointer to function if found or NULL if not found
      */
-    const Function *findFunction(const Token *tok) const;
+    const Function *findFunction(const Token *tok, bool requireConst=false) const;
 
     /**
      * @brief find if name is in nested list
@@ -619,12 +890,12 @@ public:
 
     const Scope *findRecordInNestedList(const std::string & name) const;
     Scope *findRecordInNestedList(const std::string & name) {
-        return const_cast<Scope *>(static_cast<const Scope *>(this)->findRecordInNestedList(name));
+        return const_cast<Scope *>(const_cast<const Scope *>(this)->findRecordInNestedList(name));
     }
 
     const Type* findType(const std::string& name) const;
     Type* findType(const std::string& name) {
-        return const_cast<Type*>(static_cast<const Scope *>(this)->findType(name));
+        return const_cast<Type*>(const_cast<const Scope *>(this)->findType(name));
     }
 
     /**
@@ -635,24 +906,24 @@ public:
 
     void addVariable(const Token *token_, const Token *start_,
                      const Token *end_, AccessControl access_, const Type *type_,
-                     const Scope *scope_) {
+                     const Scope *scope_, const Library* lib) {
         varlist.push_back(Variable(token_, start_, end_, varlist.size(),
                                    access_,
-                                   type_, scope_));
+                                   type_, scope_, lib));
     }
 
     /** @brief initialize varlist */
-    void getVariableList();
+    void getVariableList(const Library* lib);
 
     const Function *getDestructor() const;
 
-    /**
-     * @brief get the number of nested scopes that are not functions
-     *
-     * This returns the number of user defined types (class, struct, union)
-     * that are defined in this user defined type or namespace.
-     */
-    unsigned int getNestedNonFunctions() const;
+    void addFunction(const Function & func) {
+        functionList.push_back(func);
+
+        const Function * back = &functionList.back();
+
+        functionMap.insert(make_pair(back->tokenDef->str(), back));
+    }
 
     bool hasDefaultConstructor() const;
 
@@ -662,9 +933,10 @@ public:
      * @brief check if statement is variable declaration and add it if it is
      * @param tok pointer to start of statement
      * @param varaccess access control of statement
+     * @param lib Library instance
      * @return pointer to last token
      */
-    const Token *checkVariable(const Token *tok, AccessControl varaccess);
+    const Token *checkVariable(const Token *tok, AccessControl varaccess, const Library* lib);
 
     /**
      * @brief get variable from name
@@ -682,11 +954,14 @@ private:
      * @return true if tok points to a variable declaration, false otherwise
      */
     bool isVariableDeclaration(const Token* tok, const Token*& vartok, const Token*& typetok) const;
+
+    void findFunctionInBase(const std::string & name, size_t args, std::vector<const Function *> & matches) const;
 };
 
 class CPPCHECKLIB SymbolDatabase {
 public:
     SymbolDatabase(const Tokenizer *tokenizer, const Settings *settings, ErrorLogger *errorLogger);
+    ~SymbolDatabase();
 
     /** @brief Information about all namespaces/classes/structrues */
     std::list<Scope> scopeList;
@@ -719,23 +994,16 @@ public:
 
     const Type* findType(const Token *tok, const Scope *startScope) const;
     Type* findType(const Token *tok, Scope *startScope) const {
-        return const_cast<Type*>(this->findType(tok, static_cast<const Scope *>(startScope)));
+        return const_cast<Type*>(this->findType(tok, const_cast<const Scope *>(startScope)));
     }
 
     const Scope *findScope(const Token *tok, const Scope *startScope) const;
     Scope *findScope(const Token *tok, Scope *startScope) const {
-        return const_cast<Scope *>(this->findScope(tok, static_cast<const Scope *>(startScope)));
-    }
-
-    bool isClassOrStruct(const std::string &type) const {
-        for (std::list<Type>::const_iterator i = typeList.begin(); i != typeList.end(); ++i)
-            if (i->name() == type)
-                return true;
-        return false;
+        return const_cast<Scope *>(this->findScope(tok, const_cast<const Scope *>(startScope)));
     }
 
     const Variable *getVariableFromVarId(std::size_t varId) const {
-        return _variableList[varId];
+        return _variableList.at(varId);
     }
 
     std::size_t getVariableListSize() const {
@@ -747,22 +1015,40 @@ public:
      */
     void debugMessage(const Token *tok, const std::string &msg) const;
 
-    void printOut(const char * title = NULL) const;
+    void printOut(const char * title = nullptr) const;
     void printVariable(const Variable *var, const char *indent) const;
+    void printXml(std::ostream &out) const;
 
     bool isCPP() const;
 
-private:
+    /*
+     * @brief Do a sanity check
+     */
+    void validate() const;
 
-    // Needed by Borland C++:
+    /** Set valuetype in provided tokenlist */
+    static void setValueTypeInTokenList(Token *tokens, bool cpp, char defaultSignedness);
+
+private:
     friend class Scope;
+    friend class Function;
 
     void addClassFunction(Scope **info, const Token **tok, const Token *argStart);
     Function *addGlobalFunctionDecl(Scope*& scope, const Token* tok, const Token *argStart, const Token* funcStart);
     Function *addGlobalFunction(Scope*& scope, const Token*& tok, const Token *argStart, const Token* funcStart);
     void addNewFunction(Scope **info, const Token **tok);
-    static bool isFunction(const Token *tok, const Scope* outerScope, const Token **funcStart, const Token **argStart);
+    bool isFunction(const Token *tok, const Scope* outerScope, const Token **funcStart, const Token **argStart) const;
     const Type *findTypeInNested(const Token *tok, const Scope *startScope) const;
+    const Scope *findNamespace(const Token * tok, const Scope * scope) const;
+    Function *findFunctionInScope(const Token *func, const Scope *ns);
+    /**
+     * Send error message to error logger about internal bug.
+     * @param tok the token that this bug concerns.
+     */
+    void cppcheckError(const Token *tok) const;
+
+    /** Whether iName is a keyword as defined in http://en.cppreference.com/w/c/keyword and http://en.cppreference.com/w/cpp/keyword*/
+    bool isReservedName(const std::string& iName) const;
 
     const Tokenizer *_tokenizer;
     const Settings *_settings;
@@ -774,5 +1060,30 @@ private:
     /** list for missing types */
     std::list<Type> _blankTypes;
 };
+
+/** Value type */
+class CPPCHECKLIB ValueType {
+public:
+    enum Sign {UNKNOWN_SIGN, SIGNED, UNSIGNED} sign;
+    enum Type {UNKNOWN_TYPE, NONSTD, VOID, BOOL, CHAR, SHORT, INT, LONG, LONGLONG, FLOAT, DOUBLE, LONGDOUBLE} type;
+    unsigned int pointer; // 0=>not pointer, 1=>*, 2=>**, 3=>***, etc
+    unsigned int constness;  // bit 0=data, bit 1=*, bit 2=**
+    const Scope *typeScope;
+    std::string originalTypeName;
+
+    ValueType() : sign(UNKNOWN_SIGN), type(UNKNOWN_TYPE), pointer(0U), constness(0U), typeScope(nullptr) {}
+    ValueType(const ValueType &vt) : sign(vt.sign), type(vt.type), pointer(vt.pointer), constness(vt.constness), typeScope(vt.typeScope), originalTypeName(vt.originalTypeName) {}
+    ValueType(enum Sign s, enum Type t, unsigned int p) : sign(s), type(t), pointer(p), constness(0U), typeScope(nullptr) {}
+    ValueType(enum Sign s, enum Type t, unsigned int p, unsigned int c) : sign(s), type(t), pointer(p), constness(c), typeScope(nullptr) {}
+    ValueType(enum Sign s, enum Type t, unsigned int p, unsigned int c, const std::string &otn) : sign(s), type(t), pointer(p), constness(c), typeScope(nullptr), originalTypeName(otn) {}
+
+    bool isIntegral() const {
+        return (type >= ValueType::Type::BOOL && type <= ValueType::Type::LONGLONG);
+    }
+
+    std::string str() const;
+};
+
+
 //---------------------------------------------------------------------------
 #endif // symboldatabaseH
