@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -51,10 +51,10 @@ public:
     static void cleanupAfterSimplify(Token *tokens);
 
     /**
-     * @return 0 if there are no syntax errors or return token which identifies
-     * the location of syntax error.
+     * \param[in] tokens token list
+     * @return false if there are no syntax errors or true
      */
-    static const Token* hasComplicatedSyntaxErrorsInTemplates(Token *tokens);
+    static void checkComplicatedSyntaxErrorsInTemplates(const Token *tokens);
 
     /**
      * is the token pointing at a template parameters block
@@ -117,6 +117,19 @@ public:
         std::vector<const Token *> &typesUsedInTemplateInstantiation,
         std::list<Token *> &templateInstantiations);
 
+    /**
+     * @brief TemplateParametersInDeclaration
+     * @param tok  template < typename T, typename S >
+     *                        ^ tok
+     * @param typeParametersInDeclaration  template < typename T, typename S >
+     *                                                         ^ [0]       ^ [1]
+     * @return  template < typename T, typename S >
+     *                                              ^ return
+     */
+    static const Token * TemplateParametersInDeclaration(
+        const Token * tok,
+        std::vector<const Token *> & typeParametersInDeclaration);
+
     /**
      * Simplify templates : expand all instantiations for a template
      * @todo It seems that inner templates should be instantiated recursively
@@ -130,7 +143,7 @@ public:
      */
     static bool simplifyTemplateInstantiations(
         TokenList& tokenlist,
-        ErrorLogger& errorlogger,
+        ErrorLogger* errorlogger,
         const Settings *_settings,
         const Token *tok,
         std::list<Token *> &templateInstantiations,
@@ -145,7 +158,7 @@ public:
      */
     static void simplifyTemplates(
         TokenList& tokenlist,
-        ErrorLogger& errorlogger,
+        ErrorLogger* errorlogger,
         const Settings *_settings,
         bool &_codeWithTemplates);
 
@@ -173,6 +186,9 @@ private:
      */
     static bool removeTemplate(Token *tok);
 
+    /** Syntax error */
+    static void syntaxError(const Token *tok);
+
 };
 
 /// @}
