@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,13 +16,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
 #include "tokenize.h"
 #include "checkbool.h"
 #include "testsuite.h"
-#include <sstream>
 
-extern std::ostringstream errout;
 
 class TestBool : public TestFixture {
 public:
@@ -30,12 +27,17 @@ public:
     }
 
 private:
-
+    Settings settings;
 
     void run() {
+        settings.addEnabled("style");
+        settings.addEnabled("warning");
+        settings.inconclusive = true;
+
         TEST_CASE(bitwiseOnBoolean);      // if (bool & bool)
         TEST_CASE(incrementBoolean);
         TEST_CASE(assignBoolToPointer);
+        TEST_CASE(assignBoolToFloat);
 
         TEST_CASE(comparisonOfBoolExpressionWithInt1);
         TEST_CASE(comparisonOfBoolExpressionWithInt2);
@@ -56,6 +58,7 @@ private:
         TEST_CASE(checkComparisonOfFuncReturningBool4);
         TEST_CASE(checkComparisonOfFuncReturningBool5);
         TEST_CASE(checkComparisonOfFuncReturningBool6);
+        TEST_CASE(checkComparisonOfBoolWithBool);
 
         // Converting pointer addition result to bool
         TEST_CASE(pointerArithBool1);
@@ -65,10 +68,6 @@ private:
         // Clear the error buffer..
         errout.str("");
 
-        Settings settings;
-        settings.addEnabled("style");
-        settings.addEnabled("warning");
-        settings.inconclusive = true;
         settings.experimental = experimental;
 
         // Tokenize..
@@ -91,6 +90,21 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (error) Boolean value assigned to pointer.\n", errout.str());
 
+        check("void foo(bool *p) {\n"
+              "    p = (x<y);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (error) Boolean value assigned to pointer.\n", errout.str());
+
+        check("void foo(bool *p) {\n"
+              "    p = (x||y);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (error) Boolean value assigned to pointer.\n", errout.str());
+
+        check("void foo(bool *p) {\n"
+              "    p = (x&&y);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (error) Boolean value assigned to pointer.\n", errout.str());
+
         // check against potential false positives
         check("void foo(bool *p) {\n"
               "    *p = false;\n"
@@ -115,6 +129,64 @@ private:
               "    s.p = true;\n"
               "}\n");
         ASSERT_EQUALS("[test.cpp:6]: (error) Boolean value assigned to pointer.\n", errout.str());
+
+        // ticket #5627 - false positive: template
+        check("void f() {\n"
+              "    X *p = new ::std::pair<int,int>[rSize];\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // ticket #6588 (c mode)
+        check("struct MpegEncContext { int *q_intra_matrix, *q_chroma_intra_matrix; };\n"
+              "void dnxhd_10bit_dct_quantize(MpegEncContext *ctx, int n, int qscale) {\n"
+              "  const int *qmat = n < 4;\n" /* KO */
+              "  const int *rmat = n < 4 ? " /* OK */
+              "                       ctx->q_intra_matrix :"
+              "                       ctx->q_chroma_intra_matrix;\n"
+              "}", /*experimental=*/false, "test.c");
+        ASSERT_EQUALS("[test.c:3]: (error) Boolean value assigned to pointer.\n", errout.str());
+
+        // ticket #6588 (c++ mode)
+        check("struct MpegEncContext { int *q_intra_matrix, *q_chroma_intra_matrix; };\n"
+              "void dnxhd_10bit_dct_quantize(MpegEncContext *ctx, int n, int qscale) {\n"
+              "  const int *qmat = n < 4;\n" /* KO */
+              "  const int *rmat = n < 4 ? " /* OK */
+              "                       ctx->q_intra_matrix :"
+              "                       ctx->q_chroma_intra_matrix;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Boolean value assigned to pointer.\n", errout.str());
+
+        // ticket #6665
+        check("void pivot_big(char *first, int compare(const void *, const void *)) {\n"
+              "  char *a = first, *b = first + 1, *c = first + 2;\n"
+              "  char* m1 = compare(a, b) < 0\n"
+              "      ? (compare(b, c) < 0 ? b : (compare(a, c) < 0 ? c : a))\n"
+              "      : (compare(a, c) < 0 ? a : (compare(b, c) < 0 ? c : b));\n"
+              "}", /*experimental=*/false, "test.c");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void assignBoolToFloat() {
+        check("void foo1() {\n"
+              "    double d = false;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (style) Boolean value assigned to floating point variable.\n", errout.str());
+
+        check("void foo2() {\n"
+              "    float d = true;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (style) Boolean value assigned to floating point variable.\n", errout.str());
+
+        check("void foo3() {\n"
+              "    long double d = (2>1);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (style) Boolean value assigned to floating point variable.\n", errout.str());
+
+        // stability - don't crash:
+        check("void foo4() {\n"
+              "    unknown = false;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void comparisonOfBoolExpressionWithInt1() {
@@ -407,7 +479,7 @@ private:
         ASSERT_EQUALS("",errout.str());
 
         check("void f(int a, int b, int c) { if (1 < !(a+b)) {} }");
-        TODO_ASSERT_EQUALS("error","",errout.str());
+        ASSERT_EQUALS("[test.cpp:1]: (warning) Comparison of a boolean expression with an integer other than 0 or 1.\n",errout.str());
     }
 
     void comparisonOfBoolExpressionWithInt3() {
@@ -422,12 +494,12 @@ private:
         check("void f() {\n"
               "  for(int i = 4; i > -1 < 5 ; --i) {}\n"
               "}");
-        ASSERT_EQUALS("[test.cpp:2]: (warning) Comparison of a boolean value using relational operator (<, >, <= or >=).\n", errout.str());
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Comparison of a boolean expression with an integer other than 0 or 1.\n", errout.str());
 
         check("void f(int a, int b, int c) {\n"
               "  return (a > b) < c;\n"
               "}");
-        TODO_ASSERT_EQUALS("error", "", errout.str());
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Comparison of a boolean expression with an integer.\n", errout.str());
 
         check("void f(int a, int b, int c) {\n"
               "  return x(a > b) < c;\n"
@@ -879,10 +951,30 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (error) Converting pointer arithmetic result to bool. The bool is always true unless there is undefined behaviour.\n", errout.str());
 
+        check("void f(char *p) {\n"
+              "    do {} while (p+1);\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (error) Converting pointer arithmetic result to bool. The bool is always true unless there is undefined behaviour.\n", errout.str());
+
+        check("void f(char *p) {\n"
+              "    while (p-1) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (error) Converting pointer arithmetic result to bool. The bool is always true unless there is undefined behaviour.\n", errout.str());
+
+        check("void f(char *p) {\n"
+              "    for (int i = 0; p+1; i++) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (error) Converting pointer arithmetic result to bool. The bool is always true unless there is undefined behaviour.\n", errout.str());
+
         check("void f(char *p) {\n"
               "    if (p && p+1){}\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (error) Converting pointer arithmetic result to bool. The bool is always true unless there is undefined behaviour.\n", errout.str());
+
+        check("void f(char *p) {\n"
+              "    if (p+2 || p) {}\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (error) Converting pointer arithmetic result to bool. The bool is always true unless there is undefined behaviour.\n", errout.str());
     }
 };
 
