@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include "redirect.h"
 #include "timer.h"
 
+
 class TestCmdlineParser : public TestFixture {
 public:
     TestCmdlineParser()
@@ -71,7 +72,7 @@ private:
         TEST_CASE(enabledPortability);
         TEST_CASE(enabledUnusedFunction);
         TEST_CASE(enabledMissingInclude);
-#ifndef NDEBUG
+#ifdef CHECK_INTERNAL
         TEST_CASE(enabledInternal);
 #endif
         TEST_CASE(enabledMultiple);
@@ -203,17 +204,17 @@ private:
     void verboseshort() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-v", "file.cpp"};
-        settings._verbose = false;
+        settings.verbose = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS(true, settings._verbose);
+        ASSERT_EQUALS(true, settings.verbose);
     }
 
     void verboselong() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--verbose", "file.cpp"};
-        settings._verbose = false;
+        settings.verbose = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS(true, settings._verbose);
+        ASSERT_EQUALS(true, settings.verbose);
     }
 
     void debug() {
@@ -235,68 +236,68 @@ private:
     void forceshort() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-f", "file.cpp"};
-        settings._force = false;
+        settings.force = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS(true, settings._force);
+        ASSERT_EQUALS(true, settings.force);
     }
 
     void forcelong() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--force", "file.cpp"};
-        settings._force = false;
+        settings.force = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS(true, settings._force);
+        ASSERT_EQUALS(true, settings.force);
     }
 
     void relativePaths() {
         REDIRECT;
-        settings._relativePaths = false;
+        settings.relativePaths = false;
 
         const char *argvs[] = {"cppcheck", "-rp", "file.cpp"};
         ASSERT(defParser.ParseFromArgs(3, argvs));
-        ASSERT_EQUALS(true, settings._relativePaths);
+        ASSERT_EQUALS(true, settings.relativePaths);
 
-        settings._relativePaths = false;
+        settings.relativePaths = false;
 
         const char *argvl[] = {"cppcheck", "--relative-paths", "file.cpp"};
         ASSERT(defParser.ParseFromArgs(3, argvl));
-        ASSERT_EQUALS(true, settings._relativePaths);
+        ASSERT_EQUALS(true, settings.relativePaths);
 
-        settings._relativePaths = false;
-        settings._basePaths.clear();
+        settings.relativePaths = false;
+        settings.basePaths.clear();
 
         const char *argvsp[] = {"cppcheck", "-rp=C:/foo;C:\\bar", "file.cpp"};
         ASSERT(defParser.ParseFromArgs(3, argvsp));
-        ASSERT_EQUALS(true, settings._relativePaths);
-        ASSERT_EQUALS(2, settings._basePaths.size());
-        ASSERT_EQUALS("C:/foo", settings._basePaths[0]);
-        ASSERT_EQUALS("C:/bar", settings._basePaths[1]);
+        ASSERT_EQUALS(true, settings.relativePaths);
+        ASSERT_EQUALS(2, settings.basePaths.size());
+        ASSERT_EQUALS("C:/foo", settings.basePaths[0]);
+        ASSERT_EQUALS("C:/bar", settings.basePaths[1]);
 
-        settings._relativePaths = false;
-        settings._basePaths.clear();
+        settings.relativePaths = false;
+        settings.basePaths.clear();
 
         const char *argvlp[] = {"cppcheck", "--relative-paths=C:/foo;C:\\bar", "file.cpp"};
         ASSERT(defParser.ParseFromArgs(3, argvlp));
-        ASSERT_EQUALS(true, settings._relativePaths);
-        ASSERT_EQUALS(2, settings._basePaths.size());
-        ASSERT_EQUALS("C:/foo", settings._basePaths[0]);
-        ASSERT_EQUALS("C:/bar", settings._basePaths[1]);
+        ASSERT_EQUALS(true, settings.relativePaths);
+        ASSERT_EQUALS(2, settings.basePaths.size());
+        ASSERT_EQUALS("C:/foo", settings.basePaths[0]);
+        ASSERT_EQUALS("C:/bar", settings.basePaths[1]);
     }
 
     void quietshort() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-q", "file.cpp"};
-        settings._errorsOnly = false;
+        settings.quiet = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS(true, settings._errorsOnly);
+        ASSERT_EQUALS(true, settings.quiet);
     }
 
     void quietlong() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--quiet", "file.cpp"};
-        settings._errorsOnly = false;
+        settings.quiet = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS(true, settings._errorsOnly);
+        ASSERT_EQUALS(true, settings.quiet);
     }
 
     void defines_noarg() {
@@ -402,50 +403,50 @@ private:
     void includes() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-I", "include", "file.cpp"};
-        settings._includePaths.clear();
+        settings.includePaths.clear();
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS("include/", settings._includePaths.front());
+        ASSERT_EQUALS("include/", settings.includePaths.front());
     }
 
     void includesslash() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-I", "include/", "file.cpp"};
-        settings._includePaths.clear();
+        settings.includePaths.clear();
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS("include/", settings._includePaths.front());
+        ASSERT_EQUALS("include/", settings.includePaths.front());
     }
 
     void includesbackslash() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-I", "include\\", "file.cpp"};
-        settings._includePaths.clear();
+        settings.includePaths.clear();
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS("include/", settings._includePaths.front());
+        ASSERT_EQUALS("include/", settings.includePaths.front());
     }
 
     void includesnospace() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-Iinclude", "file.cpp"};
-        settings._includePaths.clear();
+        settings.includePaths.clear();
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS("include/", settings._includePaths.front());
+        ASSERT_EQUALS("include/", settings.includePaths.front());
     }
 
     void includes2() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-I", "include/", "-I", "framework/", "file.cpp"};
-        settings._includePaths.clear();
+        settings.includePaths.clear();
         ASSERT(defParser.ParseFromArgs(6, argv));
-        ASSERT_EQUALS("include/", settings._includePaths.front());
-        settings._includePaths.pop_front();
-        ASSERT_EQUALS("framework/", settings._includePaths.front());
+        ASSERT_EQUALS("include/", settings.includePaths.front());
+        settings.includePaths.pop_front();
+        ASSERT_EQUALS("framework/", settings.includePaths.front());
     }
 
     void includesFile() {
         // TODO: Fails since cannot open the file
         REDIRECT;
         const char *argv[] = {"cppcheck", "--includes-file=inclpaths.txt", "file.cpp"};
-        settings._includePaths.clear();
+        settings.includePaths.clear();
         ASSERT_EQUALS(true, defParser.ParseFromArgs(3, argv));
     }
 
@@ -516,7 +517,7 @@ private:
         ASSERT(settings.isEnabled("missingInclude"));
     }
 
-#ifndef NDEBUG
+#ifdef CHECK_INTERNAL
     void enabledInternal() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--enable=internal", "file.cpp"};
@@ -550,15 +551,15 @@ private:
     void errorExitcode() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--error-exitcode=5", "file.cpp"};
-        settings._exitCode = 0;
+        settings.exitCode = 0;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT_EQUALS(5, settings._exitCode);
+        ASSERT_EQUALS(5, settings.exitCode);
     }
 
     void errorExitcodeMissing() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--error-exitcode=", "file.cpp"};
-        settings._exitCode = 0;
+        settings.exitCode = 0;
         // Fails since exit code not given
         ASSERT_EQUALS(false, defParser.ParseFromArgs(3, argv));
     }
@@ -566,7 +567,7 @@ private:
     void errorExitcodeStr() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--error-exitcode=foo", "file.cpp"};
-        settings._exitCode = 0;
+        settings.exitCode = 0;
         // Fails since invalid exit code
         ASSERT_EQUALS(false, defParser.ParseFromArgs(3, argv));
     }
@@ -575,7 +576,7 @@ private:
         // TODO: Fails since cannot open the file
         REDIRECT;
         const char *argv[] = {"cppcheck", "--exitcode-suppressions", "suppr.txt", "file.cpp"};
-        settings._exitCode = 0;
+        settings.exitCode = 0;
         TODO_ASSERT_EQUALS(true, false, defParser.ParseFromArgs(4, argv));
     }
 
@@ -583,7 +584,7 @@ private:
         // TODO: Fails since cannot open the file
         REDIRECT;
         const char *argv[] = {"cppcheck", "--exitcode-suppressions=suppr.txt", "file.cpp"};
-        settings._exitCode = 0;
+        settings.exitCode = 0;
         TODO_ASSERT_EQUALS(true, false, defParser.ParseFromArgs(3, argv));
     }
 
@@ -591,7 +592,7 @@ private:
         // TODO: Fails since cannot open the file
         REDIRECT;
         const char *argv[] = {"cppcheck", "--exitcode-suppressions", "file.cpp"};
-        settings._exitCode = 0;
+        settings.exitCode = 0;
         TODO_ASSERT_EQUALS(true, false, defParser.ParseFromArgs(3, argv));
     }
 
@@ -602,13 +603,13 @@ private:
         TODO_ASSERT_EQUALS(true, false, defParser.ParseFromArgs(4, argv));
     }
 
-    void fileListStdin() {
-        // TODO: Give it some stdin to read from, fails because the list of
-        // files in stdin (_pathnames) is empty
-        REDIRECT;
-        const char *argv[] = {"cppcheck", "--file-list=-", "file.cpp"};
-        TODO_ASSERT_EQUALS(true, false, defParser.ParseFromArgs(3, argv));
-    }
+    /*    void fileListStdin() {
+            // TODO: Give it some stdin to read from, fails because the list of
+            // files in stdin (_pathnames) is empty
+            REDIRECT;
+            const char *argv[] = {"cppcheck", "--file-list=-", "file.cpp"};
+            TODO_ASSERT_EQUALS(true, false, defParser.ParseFromArgs(3, argv));
+        } */
 
     void inlineSuppr() {
         REDIRECT;
@@ -619,15 +620,15 @@ private:
     void jobs() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-j", "3", "file.cpp"};
-        settings._jobs = 0;
+        settings.jobs = 0;
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS(3, settings._jobs);
+        ASSERT_EQUALS(3, settings.jobs);
     }
 
     void jobsMissingCount() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-j", "file.cpp"};
-        settings._jobs = 0;
+        settings.jobs = 0;
         // Fails since -j is missing thread count
         ASSERT_EQUALS(false, defParser.ParseFromArgs(3, argv));
     }
@@ -635,7 +636,7 @@ private:
     void jobsInvalid() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-j", "e", "file.cpp"};
-        settings._jobs = 0;
+        settings.jobs = 0;
         // Fails since invalid count given for -j
         ASSERT_EQUALS(false, defParser.ParseFromArgs(4, argv));
     }
@@ -643,11 +644,11 @@ private:
     void maxConfigs() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "-f", "--max-configs=12", "file.cpp"};
-        settings._force = false;
-        settings._maxConfigs = 12;
+        settings.force = false;
+        settings.maxConfigs = 12;
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS(12, settings._maxConfigs);
-        ASSERT_EQUALS(false, settings._force);
+        ASSERT_EQUALS(12, settings.maxConfigs);
+        ASSERT_EQUALS(false, settings.force);
     }
 
     void maxConfigsMissingCount() {
@@ -767,102 +768,102 @@ private:
 
     void suppressionTwo() {
         REDIRECT;
-        const char *argv[] = {"cppcheck", "--suppress=uninitvar,unnecessaryQualification", "file.cpp"};
+        const char *argv[] = {"cppcheck", "--suppress=uninitvar,noConstructor", "file.cpp"};
         settings = Settings();
         TODO_ASSERT_EQUALS(true, false, defParser.ParseFromArgs(3, argv));
         TODO_ASSERT_EQUALS(true, false, settings.nomsg.isSuppressed("uninitvar", "file.cpp", 1U));
-        TODO_ASSERT_EQUALS(true, false, settings.nomsg.isSuppressed("unnecessaryQualification", "file.cpp", 1U));
+        TODO_ASSERT_EQUALS(true, false, settings.nomsg.isSuppressed("noConstructor", "file.cpp", 1U));
     }
 
     void suppressionTwoSeparate() {
         REDIRECT;
-        const char *argv[] = {"cppcheck", "--suppress=uninitvar", "--suppress=unnecessaryQualification", "file.cpp"};
+        const char *argv[] = {"cppcheck", "--suppress=uninitvar", "--suppress=noConstructor", "file.cpp"};
         settings = Settings();
         ASSERT(defParser.ParseFromArgs(4, argv));
         ASSERT_EQUALS(true, settings.nomsg.isSuppressed("uninitvar", "file.cpp", 1U));
-        ASSERT_EQUALS(true, settings.nomsg.isSuppressed("unnecessaryQualification", "file.cpp", 1U));
+        ASSERT_EQUALS(true, settings.nomsg.isSuppressed("noConstructor", "file.cpp", 1U));
     }
 
     void templates() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--template", "{file}:{line},{severity},{id},{message}", "file.cpp"};
-        settings._outputFormat.clear();
+        settings.outputFormat.clear();
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS("{file}:{line},{severity},{id},{message}", settings._outputFormat);
+        ASSERT_EQUALS("{file}:{line},{severity},{id},{message}", settings.outputFormat);
     }
 
     void templatesGcc() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--template", "gcc", "file.cpp"};
-        settings._outputFormat.clear();
+        settings.outputFormat.clear();
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS("{file}:{line}: {severity}: {message}", settings._outputFormat);
+        ASSERT_EQUALS("{file}:{line}: {severity}: {message}", settings.outputFormat);
     }
 
     void templatesVs() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--template", "vs", "file.cpp"};
-        settings._outputFormat.clear();
+        settings.outputFormat.clear();
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS("{file}({line}): {severity}: {message}", settings._outputFormat);
+        ASSERT_EQUALS("{file}({line}): {severity}: {message}", settings.outputFormat);
     }
 
     void templatesEdit() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--template", "edit", "file.cpp"};
-        settings._outputFormat.clear();
+        settings.outputFormat.clear();
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT_EQUALS("{file} +{line}: {severity}: {message}", settings._outputFormat);
+        ASSERT_EQUALS("{file} +{line}: {severity}: {message}", settings.outputFormat);
     }
 
     void xml() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--xml", "file.cpp"};
-        settings._xml_version = 1;
-        settings._xml = false;
+        settings.xml_version = 1;
+        settings.xml = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT(settings._xml);
-        ASSERT_EQUALS(1, settings._xml_version);
+        ASSERT(settings.xml);
+        ASSERT_EQUALS(1, settings.xml_version);
     }
 
     void xmlver1() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--xml-version=1", "file.cpp"};
-        settings._xml_version = 1;
-        settings._xml = false;
+        settings.xml_version = 1;
+        settings.xml = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT(settings._xml);
-        ASSERT_EQUALS(1, settings._xml_version);
+        ASSERT(settings.xml);
+        ASSERT_EQUALS(1, settings.xml_version);
     }
 
     void xmlver2() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--xml-version=2", "file.cpp"};
-        settings._xml_version = 1;
-        settings._xml = false;
+        settings.xml_version = 1;
+        settings.xml = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT(settings._xml);
-        ASSERT_EQUALS(2, settings._xml_version);
+        ASSERT(settings.xml);
+        ASSERT_EQUALS(2, settings.xml_version);
     }
 
     void xmlver2both() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--xml", "--xml-version=2", "file.cpp"};
-        settings._xml_version = 1;
-        settings._xml = false;
+        settings.xml_version = 1;
+        settings.xml = false;
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT(settings._xml);
-        ASSERT_EQUALS(2, settings._xml_version);
+        ASSERT(settings.xml);
+        ASSERT_EQUALS(2, settings.xml_version);
     }
 
     void xmlver2both2() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--xml-version=2", "--xml", "file.cpp"};
-        settings._xml_version = 1;
-        settings._xml = false;
+        settings.xml_version = 1;
+        settings.xml = false;
         ASSERT(defParser.ParseFromArgs(4, argv));
-        ASSERT(settings._xml);
-        ASSERT_EQUALS(2, settings._xml_version);
+        ASSERT(settings.xml);
+        ASSERT_EQUALS(2, settings.xml_version);
     }
 
     void xmlverunknown() {
@@ -889,9 +890,9 @@ private:
     void showtime() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--showtime=summary"};
-        settings._showtime = SHOWTIME_NONE;
+        settings.showtime = SHOWTIME_NONE;
         ASSERT(defParser.ParseFromArgs(2, argv));
-        ASSERT(settings._showtime == SHOWTIME_SUMMARY);
+        ASSERT(settings.showtime == SHOWTIME_SUMMARY);
     }
 
     void errorlist1() {
@@ -904,17 +905,17 @@ private:
     void errorlistverbose1() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--verbose", "--errorlist"};
-        settings._verbose = false;
+        settings.verbose = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT(settings._verbose);
+        ASSERT(settings.verbose);
     }
 
     void errorlistverbose2() {
         REDIRECT;
         const char *argv[] = {"cppcheck", "--errorlist", "--verbose"};
-        settings._verbose = false;
+        settings.verbose = false;
         ASSERT(defParser.ParseFromArgs(3, argv));
-        ASSERT(settings._verbose);
+        ASSERT(settings.verbose);
     }
 
     void ignorepathsnopath() {
