@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,18 +16,10 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-
-
-// Check for unused variables..
-
-
 #include "testsuite.h"
 #include "tokenize.h"
 #include "checkother.h"
 
-#include <sstream>
-
-extern std::ostringstream errout;
 
 class TestIncompleteStatement : public TestFixture {
 public:
@@ -35,13 +27,12 @@ public:
     }
 
 private:
+    Settings settings;
+
     void check(const char code[]) {
         // Clear the error buffer..
         errout.str("");
 
-        Settings settings;
-        settings.addEnabled("warning");
-
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
         std::istringstream istr(code);
@@ -54,12 +45,16 @@ private:
     }
 
     void run() {
+        settings.addEnabled("warning");
+
         TEST_CASE(test1);
         TEST_CASE(test2);
         TEST_CASE(test3);
         TEST_CASE(test4);
         TEST_CASE(test5);
+        TEST_CASE(test6);
         TEST_CASE(test_numeric);
+        TEST_CASE(void0); // #6327: No fp for statement "(void)0;"
         TEST_CASE(intarray);
         TEST_CASE(structarraynull);
         TEST_CASE(structarray);
@@ -68,6 +63,8 @@ private:
         TEST_CASE(returnstruct);
         TEST_CASE(cast);                // #3009 : (struct Foo *)123.a = 1;
         TEST_CASE(increment);           // #3251 : FP for increment
+        TEST_CASE(cpp11init);           // #5493 : int i{1};
+        TEST_CASE(block);               // ({ do_something(); 0; })
     }
 
     void test1() {
@@ -121,6 +118,15 @@ private:
         ASSERT_EQUALS("[test.cpp:3]: (warning) Redundant code: Found a statement that begins with numeric constant.\n", errout.str());
     }
 
+    void test6() {
+        // dont crash
+        check("void f() {\n"
+              "  1 == (two + three);\n"
+              "  2 != (two + three);\n"
+              "  (one + two) != (two + three);\n"
+              "}");
+    }
+
     void test_numeric() {
         check("struct P\n"
               "{\n"
@@ -138,6 +144,14 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void void0() { // #6327
+        check("void f() { (void*)0; }");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() { $0; }");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void intarray() {
         check("int arr[] = { 100/2, 1*100 };");
         ASSERT_EQUALS("", errout.str());
@@ -173,6 +187,14 @@ private:
               "}");
         ASSERT_EQUALS("", errout.str());
 
+        // #6260 - C++11 array initialization
+        check("void foo() {\n"
+              "    static const char* a[][2] {\n"
+              "        {\"b\", \"\"},\n"
+              "    };\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
         // #2482 - false positive for empty struct
         check("struct A {};");
         ASSERT_EQUALS("", errout.str());
@@ -220,6 +242,26 @@ private:
               "}");
         ASSERT_EQUALS("", errout.str());
     }
+
+    void cpp11init() {
+        check("void f() {\n"
+              "    int x{1};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void block() {
+        check("void f() {\n"
+              "    ({ do_something(); 0; });\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        check("void f() {\n"
+              "out:\n"
+              "    ({ do_something(); 0; });\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
 };
 
 REGISTER_TEST(TestIncompleteStatement)
