@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -40,18 +40,46 @@ private:
         // Path::simplifyPath()
         ASSERT_EQUALS("index.h", Path::simplifyPath("index.h"));
         ASSERT_EQUALS("index.h", Path::simplifyPath("./index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath(".//index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath(".///index.h"));
         ASSERT_EQUALS("/index.h", Path::simplifyPath("/index.h"));
         ASSERT_EQUALS("/path/", Path::simplifyPath("/path/"));
         ASSERT_EQUALS("/", Path::simplifyPath("/"));
+        ASSERT_EQUALS("/", Path::simplifyPath("/."));
+        ASSERT_EQUALS("/", Path::simplifyPath("/./"));
+        ASSERT_EQUALS("/index.h", Path::simplifyPath("/./index.h"));
+        ASSERT_EQUALS("/", Path::simplifyPath("/.//"));
+        ASSERT_EQUALS("/index.h", Path::simplifyPath("/.//index.h"));
         ASSERT_EQUALS("../index.h", Path::simplifyPath("../index.h"));
         ASSERT_EQUALS("/index.h", Path::simplifyPath("/path/../index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("./path/../index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("path/../index.h"));
+        ASSERT_EQUALS("/index.h", Path::simplifyPath("/path//../index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("./path//../index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("path//../index.h"));
+        ASSERT_EQUALS("/index.h", Path::simplifyPath("/path/..//index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("./path/..//index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("path/..//index.h"));
+        ASSERT_EQUALS("/index.h", Path::simplifyPath("/path//..//index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("./path//..//index.h"));
+        ASSERT_EQUALS("index.h", Path::simplifyPath("path//..//index.h"));
         ASSERT_EQUALS("/index.h", Path::simplifyPath("/path/../other/../index.h"));
         ASSERT_EQUALS("/index.h", Path::simplifyPath("/path/../other///././../index.h"));
+        ASSERT_EQUALS("/index.h", Path::simplifyPath("/path/../other/././..///index.h"));
+        ASSERT_EQUALS("/index.h", Path::simplifyPath("/path/../other///././..///index.h"));
         ASSERT_EQUALS("../path/index.h", Path::simplifyPath("../path/other/../index.h"));
         ASSERT_EQUALS("a/index.h", Path::simplifyPath("a/../a/index.h"));
         ASSERT_EQUALS("a/..", Path::simplifyPath("a/.."));
+        ASSERT_EQUALS("a/..", Path::simplifyPath("./a/.."));
         ASSERT_EQUALS("../../src/test.cpp", Path::simplifyPath("../../src/test.cpp"));
         ASSERT_EQUALS("../../../src/test.cpp", Path::simplifyPath("../../../src/test.cpp"));
+        ASSERT_EQUALS("src/test.cpp", Path::simplifyPath(".//src/test.cpp"));
+        ASSERT_EQUALS("src/test.cpp", Path::simplifyPath(".///src/test.cpp"));
+        ASSERT_EQUALS("test.cpp", Path::simplifyPath("./././././test.cpp"));
+
+        // Handling of UNC paths on Windows
+        ASSERT_EQUALS("//src/test.cpp", Path::simplifyPath("//src/test.cpp"));
+        ASSERT_EQUALS("//src/test.cpp", Path::simplifyPath("///src/test.cpp"));
 
         // Path::removeQuotationMarks()
         ASSERT_EQUALS("index.cpp", Path::removeQuotationMarks("index.cpp"));
