@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,10 +17,6 @@
  */
 
 
-// The preprocessor that Cppcheck uses is a bit special. Instead of generating
-// the code for a known configuration, it generates the code for each configuration.
-
-
 #include "cppcheck.h"
 #include "testsuite.h"
 #include "threadexecutor.h"
@@ -29,8 +25,6 @@
 #include <map>
 #include <string>
 
-extern std::ostringstream errout;
-extern std::ostringstream output;
 
 class TestThreadExecutor : public TestFixture {
 public:
@@ -38,6 +32,7 @@ public:
     }
 
 private:
+    Settings settings;
 
     /**
      * Execute check using n jobs for y files which are have
@@ -58,8 +53,7 @@ private:
             filemap[oss.str()] = 1;
         }
 
-        Settings settings;
-        settings._jobs = jobs;
+        settings.jobs = jobs;
         ThreadExecutor executor(filemap, settings, *this);
         for (std::map<std::string, std::size_t>::const_iterator i = filemap.begin(); i != filemap.end(); ++i)
             executor.addFileContent(i->first, data);
@@ -68,7 +62,10 @@ private:
     }
 
     void run() {
+        LOAD_LIB_2(settings.library, "std.cfg");
+
         TEST_CASE(deadlock_with_many_errors);
+        TEST_CASE(many_threads);
         TEST_CASE(no_errors_more_files);
         TEST_CASE(no_errors_less_files);
         TEST_CASE(no_errors_equal_amount_files);
@@ -88,51 +85,55 @@ private:
         check(2, 3, 3, oss.str());
     }
 
+    void many_threads() {
+        check(16, 100, 100,
+              "int main()\n"
+              "{\n"
+              "  char *a = malloc(10);\n"
+              "  return 0;\n"
+              "}");
+    }
+
     void no_errors_more_files() {
-        std::ostringstream oss;
-        oss << "int main()\n"
-            << "{\n"
-            << "  return 0;\n"
-            << "}\n";
-        check(2, 3, 0, oss.str());
+        check(2, 3, 0,
+              "int main()\n"
+              "{\n"
+              "  return 0;\n"
+              "}");
     }
 
     void no_errors_less_files() {
-        std::ostringstream oss;
-        oss << "int main()\n"
-            << "{\n"
-            << "  return 0;\n"
-            << "}\n";
-        check(2, 1, 0, oss.str());
+        check(2, 1, 0,
+              "int main()\n"
+              "{\n"
+              "  return 0;\n"
+              "}");
     }
 
     void no_errors_equal_amount_files() {
-        std::ostringstream oss;
-        oss << "int main()\n"
-            << "{\n"
-            << "  return 0;\n"
-            << "}\n";
-        check(2, 2, 0, oss.str());
+        check(2, 2, 0,
+              "int main()\n"
+              "{\n"
+              "  return 0;\n"
+              "}");
     }
 
     void one_error_less_files() {
-        std::ostringstream oss;
-        oss << "int main()\n"
-            << "{\n"
-            << "  {char *a = malloc(10);}\n"
-            << "  return 0;\n"
-            << "}\n";
-        check(2, 1, 1, oss.str());
+        check(2, 1, 1,
+              "int main()\n"
+              "{\n"
+              "  {char *a = malloc(10);}\n"
+              "  return 0;\n"
+              "}");
     }
 
     void one_error_several_files() {
-        std::ostringstream oss;
-        oss << "int main()\n"
-            << "{\n"
-            << "  {char *a = malloc(10);}\n"
-            << "  return 0;\n"
-            << "}\n";
-        check(2, 20, 20, oss.str());
+        check(2, 20, 20,
+              "int main()\n"
+              "{\n"
+              "  {char *a = malloc(10);}\n"
+              "  return 0;\n"
+              "}");
     }
 };
 
