@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2013 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2016 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -21,9 +21,6 @@
 #include "checkclass.h"
 #include "testsuite.h"
 
-#include <sstream>
-
-extern std::ostringstream errout;
 
 class TestUnusedPrivateFunction : public TestFixture {
 public:
@@ -31,7 +28,11 @@ public:
     }
 
 private:
+    Settings settings;
+
     void run() {
+        settings.addEnabled("style");
+
         TEST_CASE(test1);
         TEST_CASE(test2);
         TEST_CASE(test3);
@@ -72,15 +73,17 @@ private:
 
         TEST_CASE(multiFile);
         TEST_CASE(unknownBaseTemplate); // ticket #2580
+        TEST_CASE(hierarchie_loop); // ticket 5590
+
+        TEST_CASE(staticVariable); //ticket #5566
     }
 
 
-    void check(const char code[]) {
+    void check(const char code[], Settings::PlatformType platform = Settings::Native) {
         // Clear the error buffer..
         errout.str("");
 
-        Settings settings;
-        settings.addEnabled("style");
+        settings.platform(platform);
 
         // Tokenize..
         Tokenizer tokenizer(&settings, this);
@@ -415,7 +418,7 @@ private:
               "class A\n"
               "{\n"
               "public:\n"
-              "    A()\n"
+              "    A();\n"
               "    void a();\n"
               "private:\n"
               "    void b();\n"
@@ -517,7 +520,7 @@ private:
               "public:\n"
               "    Foo() { }\n"
               "    __property int x = {read=getx}\n"
-              "};");
+              "};", Settings::Win32A);
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -530,7 +533,7 @@ private:
               "    }\n"
               "public:\n"
               "    Foo() { }\n"
-              "};");
+              "};", Settings::Win32A);
         ASSERT_EQUALS("", errout.str());
     }
 
@@ -578,6 +581,19 @@ private:
               "    }\n"
               "};");
         ASSERT_EQUALS("[test.cpp:8]: (style) Unused private function: 'Fred::startListening'\n", errout.str());
+
+        // #5059
+        check("class Fred {\n"
+              "    void* operator new(size_t obj_size, size_t buf_size) {}\n"
+              "};");
+        TODO_ASSERT_EQUALS("[test.cpp:2]: (style) Unused private function: 'Fred::operatornew'\n", "", errout.str()); // No message for operators - we currently cannot check their usage
+
+        check("class Fred {\n"
+              "    void* operator new(size_t obj_size, size_t buf_size) {}\n"
+              "public:\n"
+              "    void* foo() { return new(size) Fred(); }\n"
+              "};");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void testDoesNotIdentifyMethodAsFirstFunctionArgument() {
@@ -692,6 +708,49 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void hierarchie_loop() {
+        check("class InfiniteB : InfiniteA {\n"
+              "    class D {\n"
+              "    };\n"
+              "};\n"
+              "namespace N {\n"
+              "    class InfiniteA : InfiniteB {\n"
+              "    };\n"
+              "}\n"
+              "class InfiniteA : InfiniteB {\n"
+              "    void foo();\n"
+              "};\n"
+              "void InfiniteA::foo() {\n"
+              "    C a;\n"
+              "}");
+
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void staticVariable() {
+        check("class Foo {\n"
+              "    static int i;\n"
+              "    static int F() const { return 1; }\n"
+              "};\n"
+              "int Foo::i = Foo::F();");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class Foo {\n"
+              "    static int i;\n"
+              "    int F() const { return 1; }\n"
+              "};\n"
+              "Foo f;\n"
+              "int Foo::i = f.F();");
+        ASSERT_EQUALS("", errout.str());
+
+        check("class Foo {\n"
+              "    static int i;\n"
+              "    static int F() const { return 1; }\n"
+              "};\n"
+              "int Foo::i = sth();"
+              "int i = F();");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unused private function: 'Foo::F'\n", errout.str());
+    }
 };
 
 REGISTER_TEST(TestUnusedPrivateFunction)
