@@ -1,7 +1,7 @@
 #!/usr/bin/python
 #
 # Cppcheck - A tool for static C/C++ code analysis
-# Copyright (C) 2007-2013 Daniel Marjamaeki and Cppcheck team.
+# Copyright (C) 2007-2016 Cppcheck team.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -24,16 +24,18 @@ import argparse
 
 
 class MatchCompiler:
-    def __init__(self, verify_mode=False):
+
+    def __init__(self, verify_mode=False, show_skipped=False):
         self._verifyMode = verify_mode
+        self._showSkipped = show_skipped
         self._reset()
 
     def _reset(self):
         self._rawMatchFunctions = []
-        self._matchStrs = {}
         self._matchFunctionCache = {}
 
-    def _generateCacheSignature(self, pattern, endToken=None, varId=None, isFindMatch=False):
+    def _generateCacheSignature(
+            self, pattern, endToken=None, varId=None, isFindMatch=False):
         sig = pattern
 
         if endToken:
@@ -53,39 +55,40 @@ class MatchCompiler:
 
         return sig
 
-    def _lookupMatchFunctionId(self, pattern, endToken=None, varId=None, isFindMatch=False):
-        signature = self._generateCacheSignature(pattern, endToken, varId, isFindMatch)
+    def _lookupMatchFunctionId(
+            self, pattern, endToken=None, varId=None, isFindMatch=False):
+        signature = self._generateCacheSignature(
+            pattern, endToken, varId, isFindMatch)
 
         if signature in self._matchFunctionCache:
             return self._matchFunctionCache[signature]
 
         return None
 
-    def _insertMatchFunctionId(self, id, pattern, endToken=None, varId=None, isFindMatch=False):
-        signature = self._generateCacheSignature(pattern, endToken, varId, isFindMatch)
+    def _insertMatchFunctionId(
+            self, id, pattern, endToken=None, varId=None, isFindMatch=False):
+        signature = self._generateCacheSignature(
+            pattern, endToken, varId, isFindMatch)
 
         # function signature should not be in the cache
-        assert(self._lookupMatchFunctionId(pattern, endToken, varId, isFindMatch) == None)
+        assert(
+            self._lookupMatchFunctionId(
+                pattern,
+                endToken,
+                varId,
+                isFindMatch) is None)
 
         self._matchFunctionCache[signature] = id
 
-    def _insertMatchStr(self, look_for):
-        prefix = 'matchStr'
-
-        # Add entry if needed
-        if look_for not in self._matchStrs:
-            pos = len(self._matchStrs) + 1
-            self._matchStrs[look_for] = pos
-
-        return prefix + str(self._matchStrs[look_for])
-
     def _compileCmd(self, tok):
         if tok == '%any%':
             return 'true'
+        elif tok == '%assign%':
+            return 'tok->isAssignmentOp()'
         elif tok == '%bool%':
             return 'tok->isBoolean()'
         elif tok == '%char%':
-            return '(tok->type()==Token::eChar)'
+            return '(tok->tokType()==Token::eChar)'
         elif tok == '%comp%':
             return 'tok->isComparisonOp()'
         elif tok == '%num%':
@@ -95,28 +98,35 @@ class MatchCompiler:
         elif tok == '%op%':
             return 'tok->isOp()'
         elif tok == '%or%':
-            return '(tok->str()==' + self._insertMatchStr('|') + ')/* | */'
+            return '(tok->tokType() == Token::eBitOp && tok->str()==MatchCompiler::makeConstString("|") )'
         elif tok == '%oror%':
-            return '(tok->str()==' + self._insertMatchStr('||') + ')/* || */'
+            return '(tok->tokType() == Token::eLogicalOp && tok->str()==MatchCompiler::makeConstString("||"))'
         elif tok == '%str%':
-            return '(tok->type()==Token::eString)'
+            return '(tok->tokType()==Token::eString)'
         elif tok == '%type%':
-            return '(tok->isName() && tok->varId()==0U && tok->str() != ' + self._insertMatchStr('delete') + '/* delete */)'
-        elif tok == '%var%':
+            return (
+                '(tok->isName() && tok->varId()==0U && !tok->isKeyword())'
+            )
+        elif tok == '%name%':
             return 'tok->isName()'
+        elif tok == '%var%':
+            return '(tok->varId() != 0)'
         elif tok == '%varid%':
             return '(tok->isName() && tok->varId()==varid)'
         elif (len(tok) > 2) and (tok[0] == "%"):
-            print ("unhandled:" + tok)
+            print("unhandled:" + tok)
 
-        return '(tok->str()==' + self._insertMatchStr(tok) + ')/* ' + tok + ' */'
+        return (
+            '(tok->str()==MatchCompiler::makeConstString("' + tok + '"))'
+        )
 
-    def _compilePattern(self, pattern, nr, varid, isFindMatch=False):
+    def _compilePattern(self, pattern, nr, varid,
+                        isFindMatch=False, tokenType="const Token"):
         ret = ''
         returnStatement = ''
 
         if isFindMatch:
-            ret = '\nconst Token *tok = start_tok;\n'
+            ret = '\n    ' + tokenType + ' * tok = start_tok;\n'
             returnStatement = 'continue;\n'
         else:
             arg2 = ''
@@ -124,7 +134,8 @@ class MatchCompiler:
                 arg2 = ', const unsigned int varid'
 
             ret = '// pattern: ' + pattern + '\n'
-            ret += 'static bool match' + str(nr) + '(const Token *tok'+arg2+') {\n'
+            ret += 'static bool match' + \
+                str(nr) + '(' + tokenType + '* tok' + arg2 + ') {\n'
             returnStatement = 'return false;\n'
 
         tokens = pattern.split(' ')
@@ -144,11 +155,11 @@ class MatchCompiler:
 
             # [abc]
             if (len(tok) > 2) and (tok[0] == '[') and (tok[-1] == ']'):
-                ret += '    if (!tok || tok->str().size()!=1U || !strchr("'+tok[1:-1]+'", tok->str()[0]))\n'
+                ret += '    if (!tok || tok->str().size()!=1U || !strchr("' + tok[1:-1] + '", tok->str()[0]))\n'
                 ret += '        ' + returnStatement
 
             # a|b|c
-            elif tok.find('|') > 0:
+            elif tok.find('|') >= 0 and tok != '||' and tok != '|' and tok != '|=':
                 tokens2 = tok.split('|')
                 logicalOp = None
                 neg = None
@@ -179,7 +190,7 @@ class MatchCompiler:
 
             # !!a
             elif tok[0:2] == "!!":
-                ret += '    if (tok && tok->str() == ' + self._insertMatchStr(tok[2:]) + ')/* ' + tok[2:] + ' */\n'
+                ret += '    if (tok && tok->str() == MatchCompiler::makeConstString("' + tok[2:] + '"))\n'
                 ret += '        ' + returnStatement
                 gotoNextToken = '    tok = tok ? tok->next() : NULL;\n'
 
@@ -199,16 +210,18 @@ class MatchCompiler:
         more_args = ''
         endCondition = ''
         if endToken:
-            more_args += ', const Token *end'
+            more_args += ', const Token * end'
             endCondition = ' && start_tok != end'
         if varId:
             more_args += ', unsigned int varid'
 
         ret = '// pattern: ' + pattern + '\n'
-        ret += 'static const Token *findmatch' + str(findmatchnr) + '(const Token *start_tok'+more_args+') {\n'
-        ret += '    for (; start_tok' + endCondition + '; start_tok = start_tok->next()) {\n'
+        ret += 'template<class T> static T * findmatch' + \
+            str(findmatchnr) + '(T * start_tok' + more_args + ') {\n'
+        ret += '    for (; start_tok' + endCondition + \
+            '; start_tok = start_tok->next()) {\n'
 
-        ret += self._compilePattern(pattern, -1, varId, True)
+        ret += self._compilePattern(pattern, -1, varId, True, 'T')
         ret += '    }\n'
         ret += '    return NULL;\n}\n'
 
@@ -236,7 +249,7 @@ class MatchCompiler:
                 parlevel -= 1
                 if parlevel == 0:
                     ret = []
-                    ret.append(line[pos1:pos+1])
+                    ret.append(line[pos1:pos + 1])
                     for arg in args:
                         ret.append(arg)
                     ret.append(line[argstart:pos])
@@ -248,6 +261,23 @@ class MatchCompiler:
 
         return None
 
+    def _isInString(self, line, pos1):
+        pos = 0
+        inString = False
+        while pos != pos1:
+            if inString:
+                if line[pos] == '\\':
+                    pos += 1
+                elif line[pos] == '"':
+                    inString = False
+            else:
+                if line[pos] == '\\':
+                    pos += 1
+                elif line[pos] == '"':
+                    inString = True
+            pos += 1
+        return inString
+
     def _parseStringComparison(self, line, pos1):
         startPos = 0
         endPos = 0
@@ -259,7 +289,7 @@ class MatchCompiler:
                     pos += 1
                 elif line[pos] == '"':
                     inString = False
-                    endPos = pos+1
+                    endPos = pos + 1
                     return (startPos, endPos)
             elif line[pos] == '"':
                 startPos = pos
@@ -268,24 +298,28 @@ class MatchCompiler:
 
         return None
 
-    def _compileVerifyTokenMatch(self, is_simplematch, verifyNumber, pattern, patternNumber, varId):
+    def _compileVerifyTokenMatch(
+            self, is_simplematch, verifyNumber, pattern, patternNumber, varId):
         more_args = ''
         if varId:
             more_args = ', const unsigned int varid'
 
-        ret = 'static bool match_verify' + str(verifyNumber) + '(const Token *tok'+more_args+') {\n'
+        ret = 'static bool match_verify' + \
+            str(verifyNumber) + '(const Token *tok' + more_args + ') {\n'
 
         origMatchName = 'Match'
         if is_simplematch:
             origMatchName = 'simpleMatch'
-            assert(varId == None)
+            assert(varId is None)
 
-        ret += '    bool res_compiled_match = match'+str(patternNumber)+'(tok'
+        ret += '    bool res_compiled_match = match' + \
+            str(patternNumber) + '(tok'
         if varId:
             ret += ', varid'
         ret += ');\n'
 
-        ret += '    bool res_parsed_match = Token::' + origMatchName + '(tok, "' + pattern + '"'
+        ret += '    bool res_parsed_match = Token::' + \
+            origMatchName + '(tok, "' + pattern + '"'
         if varId:
             ret += ', varid'
         ret += ');\n'
@@ -299,36 +333,53 @@ class MatchCompiler:
 #        ret += '            std::cout << "tok: " << tok->str();\n'
 #        ret += '        if (tok->next())\n'
 #        ret += '            std::cout << "tok next: " << tok->next()->str();\n'
-        ret += '        throw InternalError(tok, "Internal error. compiled match returned different result than parsed match");\n'
+        ret += '        throw InternalError(tok, "Internal error. compiled match returned different result than parsed match: ' + pattern + '");\n'
         ret += '    }\n'
         ret += '    return res_compiled_match;\n'
         ret += '}\n'
 
         return ret
 
-    def _replaceSpecificTokenMatch(self, is_simplematch, line, start_pos, end_pos, pattern, tok, varId):
+    def _replaceSpecificTokenMatch(
+            self, is_simplematch, line, start_pos, end_pos, pattern, tok, varId):
         more_args = ''
         if varId:
             more_args = ',' + varId
 
         # Compile function or use previously compiled one
-        patternNumber = self._lookupMatchFunctionId(pattern, None, varId, False)
+        patternNumber = self._lookupMatchFunctionId(
+            pattern, None, varId, False)
 
         if patternNumber is None:
             patternNumber = len(self._rawMatchFunctions) + 1
-            self._insertMatchFunctionId(patternNumber, pattern, None, varId, False)
-            self._rawMatchFunctions.append(self._compilePattern(pattern, patternNumber, varId))
+            self._insertMatchFunctionId(
+                patternNumber,
+                pattern,
+                None,
+                varId,
+                False)
+            self._rawMatchFunctions.append(
+                self._compilePattern(pattern, patternNumber, varId))
 
         functionName = "match"
         if self._verifyMode:
             verifyNumber = len(self._rawMatchFunctions) + 1
-            self._rawMatchFunctions.append(self._compileVerifyTokenMatch(is_simplematch, verifyNumber, pattern, patternNumber, varId))
+            self._rawMatchFunctions.append(
+                self._compileVerifyTokenMatch(
+                    is_simplematch,
+                    verifyNumber,
+                    pattern,
+                    patternNumber,
+                    varId))
 
             # inject verify function
             functionName = "match_verify"
             patternNumber = verifyNumber
 
-        return line[:start_pos]+functionName+str(patternNumber)+'('+tok+more_args+')'+line[start_pos+end_pos:]
+        return (
+            line[:start_pos] + functionName + str(
+                patternNumber) + '(' + tok + more_args + ')' + line[start_pos + end_pos:]
+        )
 
     def _replaceTokenMatch(self, line):
         while True:
@@ -344,7 +395,8 @@ class MatchCompiler:
             if res is None:
                 break
 
-            assert(len(res) == 3 or len(res) == 4)  # assert that Token::Match has either 2 or 3 arguments
+            # assert that Token::Match has either 2 or 3 arguments
+            assert(len(res) == 3 or len(res) == 4)
 
             end_pos = len(res[0])
             tok = res[1]
@@ -353,37 +405,50 @@ class MatchCompiler:
             if len(res) == 4:
                 varId = res[3]
 
-            res = re.match(r'\s*"([^"]*)"\s*$', raw_pattern)
+            res = re.match(r'\s*"((?:.|\\")*?)"\s*$', raw_pattern)
             if res is None:
+                if self._showSkipped:
+                    print("[SKIPPING] match pattern: " + raw_pattern)
                 break  # Non-const pattern - bailout
 
             pattern = res.group(1)
-            line = self._replaceSpecificTokenMatch(is_simplematch, line, pos1, end_pos, pattern, tok, varId)
+            line = self._replaceSpecificTokenMatch(
+                is_simplematch,
+                line,
+                pos1,
+                end_pos,
+                pattern,
+                tok,
+                varId)
 
         return line
 
-    def _compileVerifyTokenFindMatch(self, is_findsimplematch, verifyNumber, pattern, patternNumber, endToken, varId):
+    def _compileVerifyTokenFindMatch(
+            self, is_findsimplematch, verifyNumber, pattern, patternNumber, endToken, varId):
         more_args = ''
         if endToken:
-            more_args += ', const Token *endToken'
+            more_args += ', const Token * endToken'
         if varId:
             more_args += ', const unsigned int varid'
 
-        ret = 'static const Token *findmatch_verify' + str(verifyNumber) + '(const Token *tok'+more_args+') {\n'
+        ret = 'template < class T > static T * findmatch_verify' + \
+            str(verifyNumber) + '(T * tok' + more_args + ') {\n'
 
         origFindMatchName = 'findmatch'
         if is_findsimplematch:
-            origMatchName = 'findsimplematch'
-            assert(varId == None)
+            origFindMatchName = 'findsimplematch'
+            assert(varId is None)
 
-        ret += '    const Token *res_compiled_findmatch = findmatch'+str(patternNumber)+'(tok'
+        ret += '    T * res_compiled_findmatch = findmatch' + \
+            str(patternNumber) + '(tok'
         if endToken:
             ret += ', endToken'
         if varId:
             ret += ', varid'
         ret += ');\n'
 
-        ret += '    const Token *res_parsed_findmatch = Token::' + origFindMatchName + '(tok, "' + pattern + '"'
+        ret += '    T * res_parsed_findmatch = Token::' + \
+            origFindMatchName + '(tok, "' + pattern + '"'
         if endToken:
             ret += ', endToken'
         if varId:
@@ -394,14 +459,15 @@ class MatchCompiler:
         # Don't use assert() here, it's disabled for optimized builds.
         # We also need to verify builds in 'release' mode
         ret += '    if (res_parsed_findmatch != res_compiled_findmatch) {\n'
-        ret += '        throw InternalError(tok, "Internal error. compiled findmatch returned different result than parsed findmatch");\n'
+        ret += '        throw InternalError(tok, "Internal error. compiled findmatch returned different result than parsed findmatch: ' + pattern + '");\n'
         ret += '    }\n'
         ret += '    return res_compiled_findmatch;\n'
         ret += '}\n'
 
         return ret
 
-    def _replaceSpecificFindTokenMatch(self, is_findsimplematch, line, start_pos, end_pos, pattern, tok, endToken, varId):
+    def _replaceSpecificFindTokenMatch(
+            self, is_findsimplematch, line, start_pos, end_pos, pattern, tok, endToken, varId):
         more_args = ''
         if endToken:
             more_args += ',' + endToken
@@ -409,23 +475,44 @@ class MatchCompiler:
             more_args += ',' + varId
 
         # Compile function or use previously compiled one
-        findMatchNumber = self._lookupMatchFunctionId(pattern, endToken, varId, True)
+        findMatchNumber = self._lookupMatchFunctionId(
+            pattern, endToken, varId, True)
 
         if findMatchNumber is None:
             findMatchNumber = len(self._rawMatchFunctions) + 1
-            self._insertMatchFunctionId(findMatchNumber, pattern, endToken, varId, True)
-            self._rawMatchFunctions.append(self._compileFindPattern(pattern, findMatchNumber, endToken, varId))
+            self._insertMatchFunctionId(
+                findMatchNumber,
+                pattern,
+                endToken,
+                varId,
+                True)
+            self._rawMatchFunctions.append(
+                self._compileFindPattern(
+                    pattern,
+                    findMatchNumber,
+                    endToken,
+                    varId))
 
         functionName = "findmatch"
         if self._verifyMode:
             verifyNumber = len(self._rawMatchFunctions) + 1
-            self._rawMatchFunctions.append(self._compileVerifyTokenFindMatch(is_findsimplematch, verifyNumber, pattern, findMatchNumber, endToken, varId))
+            self._rawMatchFunctions.append(
+                self._compileVerifyTokenFindMatch(
+                    is_findsimplematch,
+                    verifyNumber,
+                    pattern,
+                    findMatchNumber,
+                    endToken,
+                    varId))
 
             # inject verify function
             functionName = "findmatch_verify"
             findMatchNumber = verifyNumber
 
-        return line[:start_pos]+functionName+str(findMatchNumber)+'('+tok+more_args+')'+line[start_pos+end_pos:]
+        return (
+            line[:start_pos] + functionName + str(
+                findMatchNumber) + '(' + tok + more_args + ') ' + line[start_pos + end_pos:]
+        )
 
     def _replaceTokenFindMatch(self, line):
         pos1 = 0
@@ -442,7 +529,9 @@ class MatchCompiler:
             if res is None:
                 break
 
-            assert(len(res) >= 3 or len(res) < 6)  # assert that Token::find(simple)match has either 2, 3 or four arguments
+            assert(len(res) >= 3 or len(res) < 6)
+            # assert that Token::find(simple)match has either 2, 3 or
+            # four arguments
 
             g0 = res[0]
             tok = res[1]
@@ -461,7 +550,8 @@ class MatchCompiler:
             #     Token *findsimplematch(const Token *tok, const char pattern[]);
             #     Token *findsimplematch(const Token *tok, const char pattern[], const Token *end);
             #     Token *findmatch(const Token *tok, const char pattern[], unsigned int varId = 0);
-            #     Token *findmatch(const Token *tok, const char pattern[], const Token *end, unsigned int varId = 0);
+            # Token *findmatch(const Token *tok, const char pattern[], const
+            # Token *end, unsigned int varId = 0);
             endToken = None
             if is_findsimplematch is True and len(res) == 4:
                 endToken = res[3]
@@ -471,35 +561,47 @@ class MatchCompiler:
                 elif varId is None and len(res) == 4:
                     endToken = res[3]
 
-            res = re.match(r'\s*"([^"]*)"\s*$', pattern)
+            res = re.match(r'\s*"((?:.|\\")*?)"\s*$', pattern)
             if res is None:
+                if self._showSkipped:
+                    print("[SKIPPING] findmatch pattern: " + pattern)
                 break  # Non-const pattern - bailout
 
             pattern = res.group(1)
-            line = self._replaceSpecificFindTokenMatch(is_findsimplematch, line, pos1, len(g0), pattern, tok, endToken, varId)
+            line = self._replaceSpecificFindTokenMatch(
+                is_findsimplematch,
+                line,
+                pos1,
+                len(g0),
+                pattern,
+                tok,
+                endToken,
+                varId)
 
         return line
 
     def _replaceCStrings(self, line):
         while True:
-            match = re.search('str\(\) (==|!=) "', line)
-            if not match:
-                match = re.search('strAt\(.+?\) (==|!=) "', line)
+            match = re.search('(==|!=) *"', line)
             if not match:
                 break
 
+            if self._isInString(line, match.start()):
+                break
+
             res = self._parseStringComparison(line, match.start())
             if res is None:
                 break
 
             startPos = res[0]
             endPos = res[1]
-            text = line[startPos+1:endPos-1]
-            line = line[:startPos] + self._insertMatchStr(text) + line[endPos:]
-
+            text = line[startPos + 1:endPos - 1]
+            line = line[:startPos] + 'MatchCompiler::makeConstStringBegin' + text + 'MatchCompiler::makeConstStringEnd' + line[endPos:]
+        line = line.replace('MatchCompiler::makeConstStringBegin', 'MatchCompiler::makeConstString("')
+        line = line.replace('MatchCompiler::makeConstStringEnd', '")')
         return line
 
-    def convertFile(self, srcname, destname):
+    def convertFile(self, srcname, destname, line_directive):
         self._reset()
 
         fin = open(srcname, "rt")
@@ -508,6 +610,7 @@ class MatchCompiler:
 
         header = '#include "token.h"\n'
         header += '#include "errorlogger.h"\n'
+        header += '#include "matchcompiler.h"\n'
         header += '#include <string>\n'
         header += '#include <cstring>\n'
         # header += '#include <iostream>\n'
@@ -525,28 +628,49 @@ class MatchCompiler:
 
             code += line
 
-        # Compute string list
-        stringList = ''
-        for match in sorted(self._matchStrs, key=self._matchStrs.get):
-            stringList += 'static const std::string matchStr' + str(self._matchStrs[match]) + '("' + match + '");\n'
-
         # Compute matchFunctions
         strFunctions = ''
         for function in self._rawMatchFunctions:
             strFunctions += function
 
+        lineno = ''
+        if line_directive:
+            lineno = '#line 1 "' + srcname + '"\n'
+
         fout = open(destname, 'wt')
-        fout.write(header+stringList+strFunctions+code)
+        fout.write(header + strFunctions + lineno + code)
         fout.close()
 
 
 def main():
     # Main program
-    build_dir = 'build'
+
+    # Argument handling
+    parser = argparse.ArgumentParser(
+        description='Compile Token::Match() calls into native C++ code')
+    parser.add_argument('--verify', action='store_true', default=False,
+                        help='verify compiled matches against on-the-fly parser. Slow!')
+    parser.add_argument('--show-skipped', action='store_true', default=False,
+                        help='show skipped (non-static) patterns')
+    parser.add_argument('--read-dir', default="lib",
+                        help='directory from which files are read')
+    parser.add_argument('--write-dir', default="build",
+                        help='directory into which files are written')
+    parser.add_argument('--prefix', default="",
+                        help='prefix for build files')
+    parser.add_argument('--line', action='store_true', default=False,
+                        help='add line directive to input files into build files')
+    parser.add_argument('file', nargs='*',
+                        help='file to complile')
+    args = parser.parse_args()
+    lib_dir = args.read_dir
+    build_dir = args.write_dir
+    line_directive = args.line
+    files = args.file
 
     # Check if we are invoked from the right place
-    if not os.path.exists('lib') and not os.path.exists('samples'):
-        print('Please invoke from the top level cppcheck source dir. Example: tools/matchcompiler.py')
+    if not os.path.exists(lib_dir):
+        print('Directory "' + lib_dir + '"not found.')
         sys.exit(-1)
 
     # Create build directory if needed
@@ -555,18 +679,21 @@ def main():
     if not os.path.isdir(build_dir):
         raise Exception(build_dir + ' is not a directory')
 
-    # Argument handling
-    parser = argparse.ArgumentParser(description='Compile Token::Match() calls into native C++ code')
-    parser.add_argument('--verify', action='store_true', default=False,
-                        help='verify compiled matches against on-the-fly parser. Slow!')
-    args = parser.parse_args()
+    mc = MatchCompiler(verify_mode=args.verify,
+                       show_skipped=args.show_skipped)
 
-    mc = MatchCompiler(verify_mode=args.verify)
+    if not files:
+        # select all *.cpp files in lib_dir
+        for f in glob.glob(lib_dir + '/*.cpp'):
+            files.append(f[len(lib_dir) + 1:])
 
-    # convert all lib/*.cpp files
-    for f in glob.glob('lib/*.cpp'):
-        print (f + ' => ' + build_dir + '/' + f[4:])
-        mc.convertFile(f, build_dir + '/'+f[4:])
+    # convert files
+    for fi in files:
+        pi = lib_dir + '/' + fi
+        fo = args.prefix + fi
+        po = build_dir + '/' + fo
+        print(pi + ' => ' + po)
+        mc.convertFile(pi, po, line_directive)
 
 if __name__ == '__main__':
     main()
