@@ -10,8 +10,10 @@ mv coverage_report devinfo/
 doxygen 2> devinfo/doxygen-errors.txt
 mv doxyoutput/html devinfo/doxyoutput
 
+cd addons
+doxygen cppcheckdata.doxyfile
+mv html ../devinfo/cppcheckdata
+cd ..
+
 # Detect duplicate code..
-~/pmd-bin-4.3/bin/cpd.sh lib/ > devinfo/cpd.txt
-
-#scp -r devinfo/ danielmarjamaki,cppcheck@web.sourceforge.net:/home/groups/c/cp/cppcheck/htdocs
-
+~/pmd-4.2.6/bin/cpd.sh lib/ > devinfo/cpd.txt
